@@ -3,8 +3,12 @@ from contextlib import suppress
 from typing import Any, Optional, Tuple
 
 from .base_protocol import BaseProtocol
-from .client_exceptions import (ClientOSError, ClientPayloadError,
-                                ServerDisconnectedError, ServerTimeoutError)
+from .client_exceptions import (
+    ClientOSError,
+    ClientPayloadError,
+    ServerDisconnectedError,
+    ServerTimeoutError,
+)
 from .helpers import BaseTimerContext
 from .http import HttpResponseParser, RawResponseMessage
 from .streams import EMPTY_PAYLOAD, DataQueue, StreamReader
@@ -172,6 +176,8 @@ class ResponseHandler(BaseProtocol,
             self._payload.set_exception(exc)
 
     def data_received(self, data: bytes) -> None:
+        self._reschedule_timeout()
+
         if not data:
             return
 
