@@ -1,10 +1,11 @@
 import warnings
 from typing import Any, Dict, Iterable, List, Optional, Set  # noqa
 
+from yarl import URL
+
 from .typedefs import LooseHeaders, StrOrURL
 from .web_response import Response
 
-
 __all__ = (
     'HTTPException',
     'HTTPError',
@@ -162,7 +163,7 @@ class _HTTPMove(HTTPRedirection):
             raise ValueError("HTTP redirects need a location to redirect to.")
         super().__init__(headers=headers, reason=reason,
                          body=body, text=text, content_type=content_type)
-        self.headers['Location'] = str(location)
+        self.headers['Location'] = str(URL(location))
         self.location = location
 
 
