@@ -11,9 +11,26 @@ from contextlib import contextmanager
 from functools import wraps
 from pathlib import Path
 from types import MappingProxyType
-from typing import (TYPE_CHECKING, Any, Awaitable, Callable, Container,  # noqa
-                    Dict, Generator, Iterable, Iterator, List, Mapping,
-                    Optional, Set, Sized, Tuple, Union, cast)
+from typing import (  # noqa
+    TYPE_CHECKING,
+    Any,
+    Awaitable,
+    Callable,
+    Container,
+    Dict,
+    Generator,
+    Iterable,
+    Iterator,
+    List,
+    Mapping,
+    Optional,
+    Set,
+    Sized,
+    Tuple,
+    Type,
+    Union,
+    cast,
+)
 
 from yarl import URL
 
@@ -22,14 +39,18 @@ from .abc import AbstractMatchInfo, AbstractRouter, AbstractView
 from .helpers import DEBUG
 from .http import HttpVersion11
 from .typedefs import PathLike
-from .web_exceptions import (HTTPException, HTTPExpectationFailed,
-                             HTTPForbidden, HTTPMethodNotAllowed, HTTPNotFound)
+from .web_exceptions import (
+    HTTPException,
+    HTTPExpectationFailed,
+    HTTPForbidden,
+    HTTPMethodNotAllowed,
+    HTTPNotFound,
+)
 from .web_fileresponse import FileResponse
 from .web_request import Request
 from .web_response import Response, StreamResponse
 from .web_routedef import AbstractRouteDef
 
-
 __all__ = ('UrlDispatcher', 'UrlMappingMatchInfo',
            'AbstractResource', 'Resource', 'PlainResource', 'DynamicResource',
            'AbstractRoute', 'ResourceRoute',
@@ -103,7 +124,7 @@ class AbstractResource(Sized, Iterable['AbstractRoute']):
 class AbstractRoute(abc.ABC):
 
     def __init__(self, method: str,
-                 handler: Union[_WebHandler, AbstractView], *,
+                 handler: Union[_WebHandler, Type[AbstractView]], *,
                  expect_handler: _ExpectHandler=None,
                  resource: AbstractResource=None) -> None:
 
@@ -134,8 +155,8 @@ class AbstractRoute(abc.ABC):
             async def handler_wrapper(request: Request) -> StreamResponse:
                 result = old_handler(request)
                 if asyncio.iscoroutine(result):
-                    result = await result
-                return result
+                    return await result
+                return result  # type: ignore
             old_handler = handler
             handler = handler_wrapper
 
@@ -276,7 +297,7 @@ class Resource(AbstractResource):
         self._routes = []  # type: List[ResourceRoute]
 
     def add_route(self, method: str,
-                  handler: Union[AbstractView, _WebHandler], *,
+                  handler: Union[Type[AbstractView], _WebHandler], *,
                   expect_handler: Optional[_ExpectHandler]=None
                   ) -> 'ResourceRoute':
 
@@ -480,7 +501,7 @@ class StaticResource(PrefixResource):
                  expect_handler: Optional[_ExpectHandler]=None,
                  chunk_size: int=256 * 1024,
                  show_index: bool=False, follow_symlinks: bool=False,
-                 append_version: bool=False)-> None:
+                 append_version: bool=False) -> None:
         super().__init__(prefix, name=name)
         try:
             directory = Path(directory)
@@ -678,7 +699,8 @@ class PrefixedSubAppResource(PrefixResource):
                 'prefix': self._prefix}
 
     async def resolve(self, request: Request) -> _Resolve:
-        if not request.url.raw_path.startswith(self._prefix):
+        if not request.url.raw_path.startswith(self._prefix + '/') and \
+                request.url.raw_path != self._prefix:
             return None, set()
         match_info = await self._app.router.resolve(request)
         match_info.add_app(self._app)
@@ -805,7 +827,7 @@ class ResourceRoute(AbstractRoute):
     """A route with resource"""
 
     def __init__(self, method: str,
-                 handler: Union[_WebHandler, AbstractView],
+                 handler: Union[_WebHandler, Type[AbstractView]],
                  resource: AbstractResource, *,
                  expect_handler: Optional[_ExpectHandler]=None) -> None:
         super().__init__(method, handler, expect_handler=expect_handler,
@@ -1005,7 +1027,7 @@ class UrlDispatcher(AbstractRouter, Mapping[str, AbstractResource]):
         return resource
 
     def add_route(self, method: str, path: str,
-                  handler: Union[_WebHandler, AbstractView],
+                  handler: Union[_WebHandler, Type[AbstractView]],
                   *, name: Optional[str]=None,
                   expect_handler: Optional[_ExpectHandler]=None
                   ) -> AbstractRoute:
@@ -1092,7 +1114,7 @@ class UrlDispatcher(AbstractRouter, Mapping[str, AbstractResource]):
         """
         return self.add_route(hdrs.METH_DELETE, path, handler, **kwargs)
 
-    def add_view(self, path: str, handler: AbstractView,
+    def add_view(self, path: str, handler: Type[AbstractView],
                  **kwargs: Any) -> AbstractRoute:
         """
         Shortcut for add_route with ANY methods for a class-based view
