@@ -3,7 +3,6 @@
 with table definition for routes
 """
 
-import asyncio
 import textwrap
 
 from aiohttp import web
@@ -19,7 +18,7 @@ async def intro(request):
     resp.content_length = len(binary)
     resp.content_type = 'text/plain'
     await resp.prepare(request)
-    resp.write(binary)
+    await resp.write(binary)
     return resp
 
 
@@ -41,12 +40,12 @@ async def hello(request):
     resp.content_length = len(answer)
     resp.content_type = 'text/plain'
     await resp.prepare(request)
-    resp.write(answer)
+    await resp.write(answer)
     await resp.write_eof()
     return resp
 
 
-async def init():
+def init():
     app = web.Application()
     app.router.add_routes([
         web.get('/', intro),
@@ -57,6 +56,5 @@ async def init():
     ])
     return app
 
-loop = asyncio.get_event_loop()
-app = loop.run_until_complete(init())
-web.run_app(app)
+
+web.run_app(init())
