@@ -16,8 +16,12 @@ from yarl import URL
 
 import aiohttp
 from aiohttp import BaseConnector, hdrs, payload
-from aiohttp.client_reqrep import (ClientRequest, ClientResponse, Fingerprint,
-                                   _merge_ssl_params)
+from aiohttp.client_reqrep import (
+    ClientRequest,
+    ClientResponse,
+    Fingerprint,
+    _merge_ssl_params,
+)
 from aiohttp.test_utils import make_mocked_coro
 
 
@@ -41,8 +45,9 @@ def buf():
 
 
 @pytest.fixture
-def protocol(loop):
+def protocol(loop, transport):
     protocol = mock.Mock()
+    protocol.transport = transport
     protocol._drain_helper.return_value = loop.create_future()
     protocol._drain_helper.return_value.set_result(None)
     return protocol
@@ -73,50 +78,58 @@ def conn(transport, protocol):
     )
 
 
-def test_method1(make_request):
+def test_method1(make_request) -> None:
     req = make_request('get', 'http://python.org/')
     assert req.method == 'GET'
 
 
-def test_method2(make_request):
+def test_method2(make_request) -> None:
     req = make_request('head', 'http://python.org/')
     assert req.method == 'HEAD'
 
 
-def test_method3(make_request):
+def test_method3(make_request) -> None:
     req = make_request('HEAD', 'http://python.org/')
     assert req.method == 'HEAD'
 
 
-def test_version_1_0(make_request):
+def test_version_1_0(make_request) -> None:
     req = make_request('get', 'http://python.org/', version='1.0')
     assert req.version == (1, 0)
 
 
-def test_version_default(make_request):
+def test_version_default(make_request) -> None:
     req = make_request('get', 'http://python.org/')
     assert req.version == (1, 1)
 
 
-def test_request_info(make_request):
+def test_request_info(make_request) -> None:
     req = make_request('get', 'http://python.org/')
     assert req.request_info == aiohttp.RequestInfo(URL('http://python.org/'),
                                                    'GET',
                                                    req.headers)
 
 
-def test_version_err(make_request):
+def test_request_info_with_fragment(make_request) -> None:
+    req = make_request('get', 'http://python.org/#urlfragment')
+    assert req.request_info == aiohttp.RequestInfo(
+        URL('http://python.org/'),
+        'GET', req.headers,
+        URL('http://python.org/#urlfragment'))
+
+
+def test_version_err(make_request) -> None:
     with pytest.raises(ValueError):
         make_request('get', 'http://python.org/', version='1.c')
 
 
-def test_https_proxy(make_request):
+def test_https_proxy(make_request) -> None:
     with pytest.raises(ValueError):
         make_request(
             'get', 'http://python.org/', proxy=URL('https://proxy.org'))
 
 
-def test_keep_alive(make_request):
+def test_keep_alive(make_request) -> None:
     req = make_request('get', 'http://python.org/', version=(0, 9))
     assert not req.keep_alive()
 
@@ -135,123 +148,153 @@ def test_keep_alive(make_request):
     assert not req.keep_alive()
 
 
-def test_host_port_default_http(make_request):
+def test_host_port_default_http(make_request) -> None:
     req = make_request('get', 'http://python.org/')
     assert req.host == 'python.org'
     assert req.port == 80
     assert not req.ssl
 
 
-def test_host_port_default_https(make_request):
+def test_host_port_default_https(make_request) -> None:
     req = make_request('get', 'https://python.org/')
     assert req.host == 'python.org'
     assert req.port == 443
     assert req.is_ssl()
 
 
-def test_host_port_nondefault_http(make_request):
+def test_host_port_nondefault_http(make_request) -> None:
     req = make_request('get', 'http://python.org:960/')
     assert req.host == 'python.org'
     assert req.port == 960
     assert not req.is_ssl()
 
 
-def test_host_port_nondefault_https(make_request):
+def test_host_port_nondefault_https(make_request) -> None:
     req = make_request('get', 'https://python.org:960/')
     assert req.host == 'python.org'
     assert req.port == 960
     assert req.is_ssl()
 
 
-def test_host_port_default_ws(make_request):
+def test_host_port_default_ws(make_request) -> None:
     req = make_request('get', 'ws://python.org/')
     assert req.host == 'python.org'
     assert req.port == 80
     assert not req.is_ssl()
 
 
-def test_host_port_default_wss(make_request):
+def test_host_port_default_wss(make_request) -> None:
     req = make_request('get', 'wss://python.org/')
     assert req.host == 'python.org'
     assert req.port == 443
     assert req.is_ssl()
 
 
-def test_host_port_nondefault_ws(make_request):
+def test_host_port_nondefault_ws(make_request) -> None:
     req = make_request('get', 'ws://python.org:960/')
     assert req.host == 'python.org'
     assert req.port == 960
     assert not req.is_ssl()
 
 
-def test_host_port_nondefault_wss(make_request):
+def test_host_port_nondefault_wss(make_request) -> None:
     req = make_request('get', 'wss://python.org:960/')
     assert req.host == 'python.org'
     assert req.port == 960
     assert req.is_ssl()
 
 
-def test_host_port_err(make_request):
+def test_host_port_none_port(make_request) -> None:
+    req = make_request('get', 'unix://localhost/path')
+    assert req.headers['Host'] == 'localhost'
+
+
+def test_host_port_err(make_request) -> None:
     with pytest.raises(ValueError):
         make_request('get', 'http://python.org:123e/')
 
 
-def test_hostname_err(make_request):
+def test_hostname_err(make_request) -> None:
     with pytest.raises(ValueError):
         make_request('get', 'http://:8080/')
 
 
-def test_host_header_host_without_port(make_request):
+def test_host_header_host_first(make_request) -> None:
+    req = make_request('get', 'http://python.org/')
+    assert list(req.headers)[0] == 'Host'
+
+
+def test_host_header_host_without_port(make_request) -> None:
     req = make_request('get', 'http://python.org/')
     assert req.headers['HOST'] == 'python.org'
 
 
-def test_host_header_host_with_default_port(make_request):
+def test_host_header_host_with_default_port(make_request) -> None:
     req = make_request('get', 'http://python.org:80/')
     assert req.headers['HOST'] == 'python.org'
 
 
-def test_host_header_host_with_nondefault_port(make_request):
+def test_host_header_host_with_nondefault_port(make_request) -> None:
     req = make_request('get', 'http://python.org:99/')
     assert req.headers['HOST'] == 'python.org:99'
 
 
-def test_host_header_host_idna_encode(make_request):
+def test_host_header_host_idna_encode(make_request) -> None:
     req = make_request('get', 'http://xn--9caa.com')
     assert req.headers['HOST'] == 'xn--9caa.com'
 
 
-def test_host_header_host_unicode(make_request):
+def test_host_header_host_unicode(make_request) -> None:
     req = make_request('get', 'http://éé.com')
     assert req.headers['HOST'] == 'xn--9caa.com'
 
 
-def test_host_header_explicit_host(make_request):
+def test_host_header_explicit_host(make_request) -> None:
     req = make_request('get', 'http://python.org/',
                        headers={'host': 'example.com'})
     assert req.headers['HOST'] == 'example.com'
 
 
-def test_host_header_explicit_host_with_port(make_request):
+def test_host_header_explicit_host_with_port(make_request) -> None:
     req = make_request('get', 'http://python.org/',
                        headers={'host': 'example.com:99'})
     assert req.headers['HOST'] == 'example.com:99'
 
 
-def test_default_loop(loop):
+def test_host_header_ipv4(make_request) -> None:
+    req = make_request('get', 'http://127.0.0.2')
+    assert req.headers['HOST'] == '127.0.0.2'
+
+
+def test_host_header_ipv6(make_request) -> None:
+    req = make_request('get', 'http://[::2]')
+    assert req.headers['HOST'] == '[::2]'
+
+
+def test_host_header_ipv4_with_port(make_request) -> None:
+    req = make_request('get', 'http://127.0.0.2:99')
+    assert req.headers['HOST'] == '127.0.0.2:99'
+
+
+def test_host_header_ipv6_with_port(make_request) -> None:
+    req = make_request('get', 'http://[::2]:99')
+    assert req.headers['HOST'] == '[::2]:99'
+
+
+def test_default_loop(loop) -> None:
     asyncio.set_event_loop(loop)
     req = ClientRequest('get', URL('http://python.org/'))
     assert req.loop is loop
 
 
-def test_default_headers_useragent(make_request):
+def test_default_headers_useragent(make_request) -> None:
     req = make_request('get', 'http://python.org/')
 
     assert 'SERVER' not in req.headers
     assert 'USER-AGENT' in req.headers
 
 
-def test_default_headers_useragent_custom(make_request):
+def test_default_headers_useragent_custom(make_request) -> None:
     req = make_request('get', 'http://python.org/',
                        headers={'user-agent': 'my custom agent'})
 
@@ -259,14 +302,14 @@ def test_default_headers_useragent_custom(make_request):
     assert 'my custom agent' == req.headers['User-Agent']
 
 
-def test_skip_default_useragent_header(make_request):
+def test_skip_default_useragent_header(make_request) -> None:
     req = make_request('get', 'http://python.org/',
                        skip_auto_headers=set([istr('user-agent')]))
 
     assert 'User-Agent' not in req.headers
 
 
-def test_headers(make_request):
+def test_headers(make_request) -> None:
     req = make_request('post', 'http://python.org/',
                        headers={'Content-Type': 'text/plain'})
 
@@ -275,85 +318,85 @@ def test_headers(make_request):
     assert req.headers['ACCEPT-ENCODING'] == 'gzip, deflate'
 
 
-def test_headers_list(make_request):
+def test_headers_list(make_request) -> None:
     req = make_request('post', 'http://python.org/',
                        headers=[('Content-Type', 'text/plain')])
     assert 'CONTENT-TYPE' in req.headers
     assert req.headers['CONTENT-TYPE'] == 'text/plain'
 
 
-def test_headers_default(make_request):
+def test_headers_default(make_request) -> None:
     req = make_request('get', 'http://python.org/',
                        headers={'ACCEPT-ENCODING': 'deflate'})
     assert req.headers['ACCEPT-ENCODING'] == 'deflate'
 
 
-def test_invalid_url(make_request):
+def test_invalid_url(make_request) -> None:
     with pytest.raises(aiohttp.InvalidURL):
         make_request('get', 'hiwpefhipowhefopw')
 
 
-def test_no_path(make_request):
+def test_no_path(make_request) -> None:
     req = make_request('get', 'http://python.org')
     assert '/' == req.url.path
 
 
-def test_ipv6_default_http_port(make_request):
+def test_ipv6_default_http_port(make_request) -> None:
     req = make_request('get', 'http://[2001:db8::1]/')
     assert req.host == '2001:db8::1'
     assert req.port == 80
     assert not req.ssl
 
 
-def test_ipv6_default_https_port(make_request):
+def test_ipv6_default_https_port(make_request) -> None:
     req = make_request('get', 'https://[2001:db8::1]/')
     assert req.host == '2001:db8::1'
     assert req.port == 443
     assert req.is_ssl()
 
 
-def test_ipv6_nondefault_http_port(make_request):
+def test_ipv6_nondefault_http_port(make_request) -> None:
     req = make_request('get', 'http://[2001:db8::1]:960/')
     assert req.host == '2001:db8::1'
     assert req.port == 960
     assert not req.is_ssl()
 
 
-def test_ipv6_nondefault_https_port(make_request):
+def test_ipv6_nondefault_https_port(make_request) -> None:
     req = make_request('get', 'https://[2001:db8::1]:960/')
     assert req.host == '2001:db8::1'
     assert req.port == 960
     assert req.is_ssl()
 
 
-def test_basic_auth(make_request):
+def test_basic_auth(make_request) -> None:
     req = make_request('get', 'http://python.org',
                        auth=aiohttp.BasicAuth('nkim', '1234'))
     assert 'AUTHORIZATION' in req.headers
     assert 'Basic bmtpbToxMjM0' == req.headers['AUTHORIZATION']
 
 
-def test_basic_auth_utf8(make_request):
+def test_basic_auth_utf8(make_request) -> None:
     req = make_request('get', 'http://python.org',
                        auth=aiohttp.BasicAuth('nkim', 'секрет', 'utf-8'))
     assert 'AUTHORIZATION' in req.headers
     assert 'Basic bmtpbTrRgdC10LrRgNC10YI=' == req.headers['AUTHORIZATION']
 
 
-def test_basic_auth_tuple_forbidden(make_request):
+def test_basic_auth_tuple_forbidden(make_request) -> None:
     with pytest.raises(TypeError):
         make_request('get', 'http://python.org',
                      auth=('nkim', '1234'))
 
 
-def test_basic_auth_from_url(make_request):
+def test_basic_auth_from_url(make_request) -> None:
     req = make_request('get', 'http://nkim:1234@python.org')
     assert 'AUTHORIZATION' in req.headers
     assert 'Basic bmtpbToxMjM0' == req.headers['AUTHORIZATION']
     assert 'python.org' == req.host
 
 
-def test_basic_auth_from_url_overriden(make_request):
+def test_basic_auth_from_url_overridden(make_request) -> None:
     req = make_request('get', 'http://garbage@python.org',
                        auth=aiohttp.BasicAuth('nkim', '1234'))
     assert 'AUTHORIZATION' in req.headers
@@ -361,49 +404,49 @@ def test_basic_auth_from_url_overriden(make_request):
     assert 'python.org' == req.host
 
 
-def test_path_is_not_double_encoded1(make_request):
+def test_path_is_not_double_encoded1(make_request) -> None:
     req = make_request('get', "http://0.0.0.0/get/test case")
     assert req.url.raw_path == "/get/test%20case"
 
 
-def test_path_is_not_double_encoded2(make_request):
+def test_path_is_not_double_encoded2(make_request) -> None:
     req = make_request('get', "http://0.0.0.0/get/test%2fcase")
     assert req.url.raw_path == "/get/test%2Fcase"
 
 
-def test_path_is_not_double_encoded3(make_request):
+def test_path_is_not_double_encoded3(make_request) -> None:
     req = make_request('get', "http://0.0.0.0/get/test%20case")
     assert req.url.raw_path == "/get/test%20case"
 
 
-def test_path_safe_chars_preserved(make_request):
+def test_path_safe_chars_preserved(make_request) -> None:
     req = make_request('get', "http://0.0.0.0/get/:=+/%2B/")
     assert req.url.path == "/get/:=+/+/"
 
 
-def test_params_are_added_before_fragment1(make_request):
+def test_params_are_added_before_fragment1(make_request) -> None:
     req = make_request('GET', "http://example.com/path#fragment",
                        params={"a": "b"})
     assert str(req.url) == "http://example.com/path?a=b"
 
 
-def test_params_are_added_before_fragment2(make_request):
+def test_params_are_added_before_fragment2(make_request) -> None:
     req = make_request('GET', "http://example.com/path?key=value#fragment",
                        params={"a": "b"})
     assert str(req.url) == "http://example.com/path?key=value&a=b"
 
 
-def test_path_not_contain_fragment1(make_request):
+def test_path_not_contain_fragment1(make_request) -> None:
     req = make_request('GET', "http://example.com/path#fragment")
     assert req.url.path == "/path"
 
 
-def test_path_not_contain_fragment2(make_request):
+def test_path_not_contain_fragment2(make_request) -> None:
     req = make_request('GET', "http://example.com/path?key=value#fragment")
     assert str(req.url) == "http://example.com/path?key=value"
 
 
-def test_cookies(make_request):
+def test_cookies(make_request) -> None:
     req = make_request('get', 'http://test.com/path',
                        cookies={'cookie1': 'val1'})
 
@@ -411,7 +454,7 @@ def test_cookies(make_request):
     assert 'cookie1=val1' == req.headers['COOKIE']
 
 
-def test_cookies_merge_with_headers(make_request):
+def test_cookies_merge_with_headers(make_request) -> None:
     req = make_request('get', 'http://test.com/path',
                        headers={'cookie': 'cookie1=val1'},
                        cookies={'cookie2': 'val2'})
@@ -419,25 +462,25 @@ def test_cookies_merge_with_headers(make_request):
     assert 'cookie1=val1; cookie2=val2' == req.headers['COOKIE']
 
 
-def test_unicode_get1(make_request):
+def test_unicode_get1(make_request) -> None:
     req = make_request('get', 'http://python.org',
                        params={'foo': 'f\xf8\xf8'})
     assert 'http://python.org/?foo=f%C3%B8%C3%B8' == str(req.url)
 
 
-def test_unicode_get2(make_request):
+def test_unicode_get2(make_request) -> None:
     req = make_request('', 'http://python.org',
                        params={'f\xf8\xf8': 'f\xf8\xf8'})
 
     assert 'http://python.org/?f%C3%B8%C3%B8=f%C3%B8%C3%B8' == str(req.url)
 
 
-def test_unicode_get3(make_request):
+def test_unicode_get3(make_request) -> None:
     req = make_request('', 'http://python.org', params={'foo': 'foo'})
     assert 'http://python.org/?foo=foo' == str(req.url)
 
 
-def test_unicode_get4(make_request):
+def test_unicode_get4(make_request) -> None:
     def join(*suffix):
         return urllib.parse.urljoin('http://python.org/', '/'.join(suffix))
 
@@ -445,7 +488,7 @@ def test_unicode_get4(make_request):
     assert 'http://python.org/%C3%B8?foo=foo' == str(req.url)
 
 
-def test_query_multivalued_param(make_request):
+def test_query_multivalued_param(make_request) -> None:
     for meth in ClientRequest.ALL_METHODS:
         req = make_request(
             meth, 'http://python.org',
@@ -454,38 +497,38 @@ def test_query_multivalued_param(make_request):
         assert str(req.url) == 'http://python.org/?test=foo&test=baz'
 
 
-def test_query_str_param(make_request):
+def test_query_str_param(make_request) -> None:
     for meth in ClientRequest.ALL_METHODS:
         req = make_request(meth, 'http://python.org', params='test=foo')
         assert str(req.url) == 'http://python.org/?test=foo'
 
 
-def test_query_bytes_param_raises(make_request):
+def test_query_bytes_param_raises(make_request) -> None:
     for meth in ClientRequest.ALL_METHODS:
         with pytest.raises(TypeError):
             make_request(meth, 'http://python.org', params=b'test=foo')
 
 
-def test_query_str_param_is_not_encoded(make_request):
+def test_query_str_param_is_not_encoded(make_request) -> None:
     for meth in ClientRequest.ALL_METHODS:
         req = make_request(meth, 'http://python.org', params='test=f+oo')
         assert str(req.url) == 'http://python.org/?test=f+oo'
 
 
-def test_params_update_path_and_url(make_request):
+def test_params_update_path_and_url(make_request) -> None:
     req = make_request('get', 'http://python.org',
                        params=(('test', 'foo'), ('test', 'baz')))
     assert str(req.url) == 'http://python.org/?test=foo&test=baz'
 
 
-def test_params_empty_path_and_url(make_request):
+def test_params_empty_path_and_url(make_request) -> None:
     req_empty = make_request('get', 'http://python.org', params={})
     assert str(req_empty.url) == 'http://python.org'
     req_none = make_request('get', 'http://python.org')
     assert str(req_none.url) == 'http://python.org'
 
 
-def test_gen_netloc_all(make_request):
+def test_gen_netloc_all(make_request) -> None:
     req = make_request('get',
                        'https://aiohttp:pwpwpw@' +
                        '12345678901234567890123456789' +
@@ -494,7 +537,7 @@ def test_gen_netloc_all(make_request):
         '012345678901234567890:8080'
 
 
-def test_gen_netloc_no_port(make_request):
+def test_gen_netloc_no_port(make_request) -> None:
     req = make_request('get',
                        'https://aiohttp:pwpwpw@' +
                        '12345678901234567890123456789' +
@@ -503,7 +546,7 @@ def test_gen_netloc_no_port(make_request):
         '012345678901234567890'
 
 
-async def test_connection_header(loop, conn):
+async def test_connection_header(loop, conn) -> None:
     req = ClientRequest('get', URL('http://python.org'), loop=loop)
     req.keep_alive = mock.Mock()
     req.headers.clear()
@@ -526,7 +569,7 @@ async def test_connection_header(loop, conn):
     assert req.headers.get('CONNECTION') == 'close'
 
 
-async def test_no_content_length(loop, conn):
+async def test_no_content_length(loop, conn) -> None:
     req = ClientRequest('get', URL('http://python.org'), loop=loop)
     resp = await req.send(conn)
     assert req.headers.get('CONTENT-LENGTH') is None
@@ -534,7 +577,7 @@ async def test_no_content_length(loop, conn):
     resp.close()
 
 
-async def test_no_content_length_head(loop, conn):
+async def test_no_content_length_head(loop, conn) -> None:
     req = ClientRequest('head', URL('http://python.org'), loop=loop)
     resp = await req.send(conn)
     assert req.headers.get('CONTENT-LENGTH') is None
@@ -542,14 +585,14 @@ async def test_no_content_length_head(loop, conn):
     resp.close()
 
 
-async def test_content_type_auto_header_get(loop, conn):
+async def test_content_type_auto_header_get(loop, conn) -> None:
     req = ClientRequest('get', URL('http://python.org'), loop=loop)
     resp = await req.send(conn)
     assert 'CONTENT-TYPE' not in req.headers
     resp.close()
 
 
-async def test_content_type_auto_header_form(loop, conn):
+async def test_content_type_auto_header_form(loop, conn) -> None:
     req = ClientRequest('post', URL('http://python.org'),
                         data={'hey': 'you'}, loop=loop)
     resp = await req.send(conn)
@@ -558,7 +601,7 @@ async def test_content_type_auto_header_form(loop, conn):
     resp.close()
 
 
-async def test_content_type_auto_header_bytes(loop, conn):
+async def test_content_type_auto_header_bytes(loop, conn) -> None:
     req = ClientRequest('post', URL('http://python.org'), data=b'hey you',
                         loop=loop)
     resp = await req.send(conn)
@@ -566,7 +609,7 @@ async def test_content_type_auto_header_bytes(loop, conn):
     resp.close()
 
 
-async def test_content_type_skip_auto_header_bytes(loop, conn):
+async def test_content_type_skip_auto_header_bytes(loop, conn) -> None:
     req = ClientRequest('post', URL('http://python.org'), data=b'hey you',
                         skip_auto_headers={'Content-Type'},
                         loop=loop)
@@ -575,7 +618,7 @@ async def test_content_type_skip_auto_header_bytes(loop, conn):
     resp.close()
 
 
-async def test_content_type_skip_auto_header_form(loop, conn):
+async def test_content_type_skip_auto_header_form(loop, conn) -> None:
     req = ClientRequest('post', URL('http://python.org'),
                         data={'hey': 'you'}, loop=loop,
                         skip_auto_headers={'Content-Type'})
@@ -584,7 +627,8 @@ async def test_content_type_skip_auto_header_form(loop, conn):
     resp.close()
 
 
-async def test_content_type_auto_header_content_length_no_skip(loop, conn):
+async def test_content_type_auto_header_content_length_no_skip(loop,
+                                                               conn) -> None:
     req = ClientRequest('post', URL('http://python.org'),
                         data=io.BytesIO(b'hey'),
                         skip_auto_headers={'Content-Length'},
@@ -594,7 +638,7 @@ async def test_content_type_auto_header_content_length_no_skip(loop, conn):
     resp.close()
 
 
-async def test_urlencoded_formdata_charset(loop, conn):
+async def test_urlencoded_formdata_charset(loop, conn) -> None:
     req = ClientRequest(
         'post', URL('http://python.org'),
         data=aiohttp.FormData({'hey': 'you'}, charset='koi8-r'), loop=loop)
@@ -603,7 +647,7 @@ async def test_urlencoded_formdata_charset(loop, conn):
         req.headers.get('CONTENT-TYPE')
 
 
-async def test_post_data(loop, conn):
+async def test_post_data(loop, conn) -> None:
     for meth in ClientRequest.POST_METHODS:
         req = ClientRequest(
             meth, URL('http://python.org/'),
@@ -617,7 +661,7 @@ async def test_post_data(loop, conn):
         resp.close()
 
 
-async def test_pass_falsy_data(loop):
+async def test_pass_falsy_data(loop) -> None:
     with mock.patch(
             'aiohttp.client_reqrep.ClientRequest.update_body_from_data'):
         req = ClientRequest(
@@ -627,7 +671,7 @@ async def test_pass_falsy_data(loop):
     await req.close()
 
 
-async def test_pass_falsy_data_file(loop, tmpdir):
+async def test_pass_falsy_data_file(loop, tmpdir) -> None:
     testfile = tmpdir.join('tmpfile').open('w+b')
     testfile.write(b'data')
     testfile.seek(0)
@@ -642,7 +686,7 @@ async def test_pass_falsy_data_file(loop, tmpdir):
 
 
 # Elasticsearch API requires to send request body with GET-requests
-async def test_get_with_data(loop):
+async def test_get_with_data(loop) -> None:
     for meth in ClientRequest.GET_METHODS:
         req = ClientRequest(
             meth, URL('http://python.org/'), data={'life': '42'},
@@ -652,7 +696,7 @@ async def test_get_with_data(loop):
         await req.close()
 
 
-async def test_bytes_data(loop, conn):
+async def test_bytes_data(loop, conn) -> None:
     for meth in ClientRequest.POST_METHODS:
         req = ClientRequest(
             meth, URL('http://python.org/'),
@@ -666,7 +710,7 @@ async def test_bytes_data(loop, conn):
         resp.close()
 
 
-async def test_content_encoding(loop, conn):
+async def test_content_encoding(loop, conn) -> None:
     req = ClientRequest('post', URL('http://python.org/'), data='foo',
                         compress='deflate', loop=loop)
     with mock.patch('aiohttp.client_reqrep.StreamWriter') as m_writer:
@@ -680,7 +724,8 @@ async def test_content_encoding(loop, conn):
     resp.close()
 
 
-async def test_content_encoding_dont_set_headers_if_no_body(loop, conn):
+async def test_content_encoding_dont_set_headers_if_no_body(loop,
+                                                            conn) -> None:
     req = ClientRequest('post', URL('http://python.org/'),
                         compress='deflate', loop=loop)
     with mock.patch('aiohttp.client_reqrep.http'):
@@ -691,7 +736,7 @@ async def test_content_encoding_dont_set_headers_if_no_body(loop, conn):
     resp.close()
 
 
-async def test_content_encoding_header(loop, conn):
+async def test_content_encoding_header(loop, conn) -> None:
     req = ClientRequest(
         'post', URL('http://python.org/'), data='foo',
         headers={'Content-Encoding': 'deflate'}, loop=loop)
@@ -705,14 +750,14 @@ async def test_content_encoding_header(loop, conn):
     resp.close()
 
 
-async def test_compress_and_content_encoding(loop, conn):
+async def test_compress_and_content_encoding(loop, conn) -> None:
     with pytest.raises(ValueError):
         ClientRequest('post', URL('http://python.org/'), data='foo',
                       headers={'content-encoding': 'deflate'},
                       compress='deflate', loop=loop)
 
 
-async def test_chunked(loop, conn):
+async def test_chunked(loop, conn) -> None:
     req = ClientRequest(
         'post', URL('http://python.org/'),
         headers={'TRANSFER-ENCODING': 'gzip'}, loop=loop)
@@ -722,7 +767,7 @@ async def test_chunked(loop, conn):
     resp.close()
 
 
-async def test_chunked2(loop, conn):
+async def test_chunked2(loop, conn) -> None:
     req = ClientRequest(
         'post', URL('http://python.org/'),
         headers={'Transfer-encoding': 'chunked'}, loop=loop)
@@ -732,7 +777,7 @@ async def test_chunked2(loop, conn):
     resp.close()
 
 
-async def test_chunked_explicit(loop, conn):
+async def test_chunked_explicit(loop, conn) -> None:
     req = ClientRequest(
         'post', URL('http://python.org/'), chunked=True, loop=loop)
     with mock.patch('aiohttp.client_reqrep.StreamWriter') as m_writer:
@@ -745,23 +790,23 @@ async def test_chunked_explicit(loop, conn):
     resp.close()
 
 
-async def test_chunked_length(loop, conn):
+async def test_chunked_length(loop, conn) -> None:
     with pytest.raises(ValueError):
         ClientRequest(
             'post', URL('http://python.org/'),
             headers={'CONTENT-LENGTH': '1000'}, chunked=True, loop=loop)
 
 
-async def test_chunked_transfer_encoding(loop, conn):
+async def test_chunked_transfer_encoding(loop, conn) -> None:
     with pytest.raises(ValueError):
         ClientRequest(
             'post', URL('http://python.org/'),
             headers={'TRANSFER-ENCODING': 'chunked'}, chunked=True, loop=loop)
 
 
-async def test_file_upload_not_chunked(loop):
+async def test_file_upload_not_chunked(loop) -> None:
     here = os.path.dirname(__file__)
-    fname = os.path.join(here, 'sample.key')
+    fname = os.path.join(here, 'aiohttp.png')
     with open(fname, 'rb') as f:
         req = ClientRequest(
             'post', URL('http://python.org/'),
@@ -772,7 +817,7 @@ async def test_file_upload_not_chunked(loop):
         await req.close()
 
 
-async def test_precompressed_data_stays_intact(loop):
+async def test_precompressed_data_stays_intact(loop) -> None:
     data = zlib.compress(b'foobar')
     req = ClientRequest(
         'post', URL('http://python.org/'),
@@ -786,9 +831,9 @@ async def test_precompressed_data_stays_intact(loop):
     await req.close()
 
 
-async def test_file_upload_not_chunked_seek(loop):
+async def test_file_upload_not_chunked_seek(loop) -> None:
     here = os.path.dirname(__file__)
-    fname = os.path.join(here, 'sample.key')
+    fname = os.path.join(here, 'aiohttp.png')
     with open(fname, 'rb') as f:
         f.seek(100)
         req = ClientRequest(
@@ -800,9 +845,9 @@ async def test_file_upload_not_chunked_seek(loop):
         await req.close()
 
 
-async def test_file_upload_force_chunked(loop):
+async def test_file_upload_force_chunked(loop) -> None:
     here = os.path.dirname(__file__)
-    fname = os.path.join(here, 'sample.key')
+    fname = os.path.join(here, 'aiohttp.png')
     with open(fname, 'rb') as f:
         req = ClientRequest(
             'post', URL('http://python.org/'),
@@ -814,7 +859,7 @@ async def test_file_upload_force_chunked(loop):
         await req.close()
 
 
-async def test_expect100(loop, conn):
+async def test_expect100(loop, conn) -> None:
     req = ClientRequest('get', URL('http://python.org/'),
                         expect100=True, loop=loop)
     resp = await req.send(conn)
@@ -824,7 +869,7 @@ async def test_expect100(loop, conn):
     resp.close()
 
 
-async def test_expect_100_continue_header(loop, conn):
+async def test_expect_100_continue_header(loop, conn) -> None:
     req = ClientRequest('get', URL('http://python.org/'),
                         headers={'expect': '100-continue'}, loop=loop)
     resp = await req.send(conn)
@@ -834,7 +879,7 @@ async def test_expect_100_continue_header(loop, conn):
     resp.close()
 
 
-async def test_data_stream(loop, buf, conn):
+async def test_data_stream(loop, buf, conn) -> None:
     @async_generator
     async def gen():
         await yield_(b'binary data')
@@ -854,7 +899,7 @@ async def test_data_stream(loop, buf, conn):
     await req.close()
 
 
-async def test_data_stream_deprecated(loop, buf, conn):
+async def test_data_stream_deprecated(loop, buf, conn) -> None:
     with pytest.warns(DeprecationWarning):
         @aiohttp.streamer
         async def gen(writer):
@@ -875,7 +920,7 @@ async def test_data_stream_deprecated(loop, buf, conn):
     await req.close()
 
 
-async def test_data_file(loop, buf, conn):
+async def test_data_file(loop, buf, conn) -> None:
     req = ClientRequest(
         'POST', URL('http://python.org/'),
         data=io.BufferedReader(io.BytesIO(b'*' * 2)),
@@ -893,7 +938,7 @@ async def test_data_file(loop, buf, conn):
     await req.close()
 
 
-async def test_data_stream_exc(loop, conn):
+async def test_data_stream_exc(loop, conn) -> None:
     fut = loop.create_future()
 
     @async_generator
@@ -907,7 +952,7 @@ async def test_data_stream_exc(loop, conn):
     assert req.headers['TRANSFER-ENCODING'] == 'chunked'
 
     async def throw_exc():
-        await asyncio.sleep(0.01, loop=loop)
+        await asyncio.sleep(0.01)
         fut.set_exception(ValueError)
 
     loop.create_task(throw_exc())
@@ -919,7 +964,7 @@ async def test_data_stream_exc(loop, conn):
     await req.close()
 
 
-async def test_data_stream_exc_deprecated(loop, conn):
+async def test_data_stream_exc_deprecated(loop, conn) -> None:
     fut = loop.create_future()
 
     with pytest.warns(DeprecationWarning):
@@ -946,7 +991,7 @@ async def test_data_stream_exc_deprecated(loop, conn):
     await req.close()
 
 
-async def test_data_stream_exc_chain(loop, conn):
+async def test_data_stream_exc_chain(loop, conn) -> None:
     fut = loop.create_future()
 
     @async_generator
@@ -959,7 +1004,7 @@ async def test_data_stream_exc_chain(loop, conn):
     inner_exc = ValueError()
 
     async def throw_exc():
-        await asyncio.sleep(0.01, loop=loop)
+        await asyncio.sleep(0.01)
         fut.set_exception(inner_exc)
 
     loop.create_task(throw_exc())
@@ -975,7 +1020,7 @@ async def test_data_stream_exc_chain(loop, conn):
     await req.close()
 
 
-async def test_data_stream_exc_chain_deprecated(loop, conn):
+async def test_data_stream_exc_chain_deprecated(loop, conn) -> None:
     fut = loop.create_future()
 
     with pytest.warns(DeprecationWarning):
@@ -1005,7 +1050,7 @@ async def test_data_stream_exc_chain_deprecated(loop, conn):
     await req.close()
 
 
-async def test_data_stream_continue(loop, buf, conn):
+async def test_data_stream_continue(loop, buf, conn) -> None:
     @async_generator
     async def gen():
         await yield_(b'binary data')
@@ -1017,7 +1062,7 @@ async def test_data_stream_continue(loop, buf, conn):
     assert req.chunked
 
     async def coro():
-        await asyncio.sleep(0.0001, loop=loop)
+        await asyncio.sleep(0.0001)
         req._continue.set_result(1)
 
     loop.create_task(coro())
@@ -1030,7 +1075,7 @@ async def test_data_stream_continue(loop, buf, conn):
     resp.close()
 
 
-async def test_data_stream_continue_deprecated(loop, buf, conn):
+async def test_data_stream_continue_deprecated(loop, buf, conn) -> None:
     with pytest.warns(DeprecationWarning):
         @aiohttp.streamer
         async def gen(writer):
@@ -1057,13 +1102,13 @@ async def test_data_stream_continue_deprecated(loop, buf, conn):
     resp.close()
 
 
-async def test_data_continue(loop, buf, conn):
+async def test_data_continue(loop, buf, conn) -> None:
     req = ClientRequest(
         'POST', URL('http://python.org/'), data=b'data',
         expect100=True, loop=loop)
 
     async def coro():
-        await asyncio.sleep(0.0001, loop=loop)
+        await asyncio.sleep(0.0001)
         req._continue.set_result(1)
 
     loop.create_task(coro())
@@ -1076,7 +1121,7 @@ async def test_data_continue(loop, buf, conn):
     resp.close()
 
 
-async def test_close(loop, buf, conn):
+async def test_close(loop, buf, conn) -> None:
     @async_generator
     async def gen():
         await asyncio.sleep(0.00001)
@@ -1091,7 +1136,7 @@ async def test_close(loop, buf, conn):
     resp.close()
 
 
-async def test_close_deprecated(loop, buf, conn):
+async def test_close_deprecated(loop, buf, conn) -> None:
     with pytest.warns(DeprecationWarning):
         @aiohttp.streamer
         async def gen(writer):
@@ -1107,7 +1152,7 @@ async def test_close_deprecated(loop, buf, conn):
     resp.close()
 
 
-async def test_custom_response_class(loop, conn):
+async def test_custom_response_class(loop, conn) -> None:
     class CustomResponse(ClientResponse):
         def read(self, decode=False):
             return 'customized!'
@@ -1121,7 +1166,7 @@ async def test_custom_response_class(loop, conn):
     resp.close()
 
 
-async def test_oserror_on_write_bytes(loop, conn):
+async def test_oserror_on_write_bytes(loop, conn) -> None:
     req = ClientRequest(
         'POST', URL('http://python.org/'), loop=loop)
 
@@ -1135,7 +1180,7 @@ async def test_oserror_on_write_bytes(loop, conn):
     assert isinstance(exc, aiohttp.ClientOSError)
 
 
-async def test_terminate(loop, conn):
+async def test_terminate(loop, conn) -> None:
     req = ClientRequest('get', URL('http://python.org'), loop=loop)
     resp = await req.send(conn)
     assert req._writer is not None
@@ -1147,7 +1192,7 @@ async def test_terminate(loop, conn):
     resp.close()
 
 
-def test_terminate_with_closed_loop(loop, conn):
+def test_terminate_with_closed_loop(loop, conn) -> None:
     req = resp = writer = None
 
     async def go():
@@ -1168,7 +1213,7 @@ def test_terminate_with_closed_loop(loop, conn):
     resp.close()
 
 
-def test_terminate_without_writer(loop):
+def test_terminate_without_writer(loop) -> None:
     req = ClientRequest('get', URL('http://python.org'), loop=loop)
     assert req._writer is None
 
@@ -1176,7 +1221,7 @@ def test_terminate_without_writer(loop):
     assert req._writer is None
 
 
-async def test_custom_req_rep(loop):
+async def test_custom_req_rep(loop) -> None:
     conn = None
 
     class CustomResponse(ClientResponse):
@@ -1186,7 +1231,7 @@ async def test_custom_req_rep(loop):
             conn = connection
             self.status = 123
             self.reason = 'Test OK'
-            self.headers = CIMultiDictProxy(CIMultiDict())
+            self._headers = CIMultiDictProxy(CIMultiDict())
             self.cookies = SimpleCookie()
             return
 
@@ -1201,7 +1246,6 @@ async def test_custom_req_rep(loop):
                                        continue100=self._continue,
                                        timer=self._timer,
                                        request_info=self.request_info,
-                                       auto_decompress=self._auto_decompress,
                                        traces=self._traces,
                                        loop=self.loop,
                                        session=self._session)
@@ -1210,7 +1254,7 @@ async def test_custom_req_rep(loop):
             called = True
             return resp
 
-    async def create_connection(req, traces=None):
+    async def create_connection(req, traces, timeout):
         assert isinstance(req, CustomRequest)
         return mock.Mock()
     connector = BaseConnector(loop=loop)
@@ -1231,23 +1275,23 @@ async def test_custom_req_rep(loop):
     conn.close()
 
 
-def test_verify_ssl_false_with_ssl_context(loop):
+def test_verify_ssl_false_with_ssl_context(loop, ssl_ctx) -> None:
     with pytest.warns(DeprecationWarning):
         with pytest.raises(ValueError):
             _merge_ssl_params(None, verify_ssl=False,
-                              ssl_context=mock.Mock(), fingerprint=None)
+                              ssl_context=ssl_ctx, fingerprint=None)
 
 
-def test_bad_fingerprint(loop):
+def test_bad_fingerprint(loop) -> None:
     with pytest.raises(ValueError):
         Fingerprint(b'invalid')
 
 
-def test_insecure_fingerprint_md5(loop):
+def test_insecure_fingerprint_md5(loop) -> None:
     with pytest.raises(ValueError):
         Fingerprint(hashlib.md5(b"foo").digest())
 
 
-def test_insecure_fingerprint_sha1(loop):
+def test_insecure_fingerprint_sha1(loop) -> None:
     with pytest.raises(ValueError):
         Fingerprint(hashlib.sha1(b"foo").digest())
