@@ -8,6 +8,7 @@ from http.cookies import SimpleCookie
 from unittest import mock
 
 import pytest
+from freezegun import freeze_time
 from yarl import URL
 
 from aiohttp import CookieJar, DummyCookieJar
@@ -15,6 +16,32 @@ from aiohttp import CookieJar, DummyCookieJar
 
 @pytest.fixture
 def cookies_to_send():
+    return SimpleCookie(
+        "shared-cookie=first; "
+        "domain-cookie=second; Domain=example.com; "
+        "subdomain1-cookie=third; Domain=test1.example.com; "
+        "subdomain2-cookie=fourth; Domain=test2.example.com; "
+        "dotted-domain-cookie=fifth; Domain=.example.com; "
+        "different-domain-cookie=sixth; Domain=different.org; "
+        "secure-cookie=seventh; Domain=secure.com; Secure; "
+        "no-path-cookie=eighth; Domain=pathtest.com; "
+        "path1-cookie=nineth; Domain=pathtest.com; Path=/; "
+        "path2-cookie=tenth; Domain=pathtest.com; Path=/one; "
+        "path3-cookie=eleventh; Domain=pathtest.com; Path=/one/two; "
+        "path4-cookie=twelfth; Domain=pathtest.com; Path=/one/two/; "
+        "expires-cookie=thirteenth; Domain=expirestest.com; Path=/;"
+        " Expires=Tue, 1 Jan 2039 12:00:00 GMT; "
+        "max-age-cookie=fourteenth; Domain=maxagetest.com; Path=/;"
+        " Max-Age=60; "
+        "invalid-max-age-cookie=fifteenth; Domain=invalid-values.com; "
+        " Max-Age=string; "
+        "invalid-expires-cookie=sixteenth; Domain=invalid-values.com; "
+        " Expires=string;"
+    )
+
+
+@pytest.fixture
+def cookies_to_send_with_expired():
     return SimpleCookie(
         "shared-cookie=first; "
         "domain-cookie=second; Domain=example.com; "
@@ -54,7 +81,7 @@ def cookies_to_receive():
     )
 
 
-def test_date_parsing():
+def test_date_parsing() -> None:
     parse_func = CookieJar._parse_date
     utc = datetime.timezone.utc
 
@@ -98,7 +125,7 @@ def test_date_parsing():
     assert parse_func("Tue, 1 Jan 1970 77:88:99 GMT") is None
 
 
-def test_domain_matching():
+def test_domain_matching() -> None:
     test_func = CookieJar._is_domain_match
 
     assert test_func("test.com", "test.com")
@@ -111,7 +138,7 @@ def test_domain_matching():
     assert not test_func("test.com", "127.0.0.1")
 
 
-def test_path_matching():
+def test_path_matching() -> None:
     test_func = CookieJar._is_path_match
 
     assert test_func("/", "")
@@ -133,7 +160,7 @@ def test_path_matching():
     assert not test_func("/different-folder/", "/folder/")
 
 
-def test_constructor(loop, cookies_to_send, cookies_to_receive):
+async def test_constructor(loop, cookies_to_send, cookies_to_receive) -> None:
     jar = CookieJar(loop=loop)
     jar.update_cookies(cookies_to_send)
     jar_cookies = SimpleCookie()
@@ -144,7 +171,19 @@ def test_constructor(loop, cookies_to_send, cookies_to_receive):
     assert jar._loop is loop
 
 
-def test_save_load(loop, cookies_to_send, cookies_to_receive):
+async def test_constructor_with_expired(loop, cookies_to_send_with_expired,
+                                        cookies_to_receive) -> None:
+    jar = CookieJar()
+    jar.update_cookies(cookies_to_send_with_expired)
+    jar_cookies = SimpleCookie()
+    for cookie in jar:
+        dict.__setitem__(jar_cookies, cookie.key, cookie)
+    expected_cookies = cookies_to_send_with_expired
+    assert jar_cookies != expected_cookies
+    assert jar._loop is loop
+
+
+async def test_save_load(loop, cookies_to_send, cookies_to_receive) -> None:
     file_path = tempfile.mkdtemp() + '/aiohttp.test.cookie'
 
     # export cookie jar
@@ -163,7 +202,7 @@ def test_save_load(loop, cookies_to_send, cookies_to_receive):
     assert jar_test == cookies_to_receive
 
 
-def test_update_cookie_with_unicode_domain(loop):
+async def test_update_cookie_with_unicode_domain(loop) -> None:
     cookies = (
         "idna-domain-first=first; Domain=xn--9caa.com; Path=/;",
         "idna-domain-second=second; Domain=xn--9caa.com; Path=/;",
@@ -180,8 +219,8 @@ def test_update_cookie_with_unicode_domain(loop):
     assert jar_test == SimpleCookie(" ".join(cookies))
 
 
-def test_filter_cookie_with_unicode_domain(loop):
-    jar = CookieJar(loop=loop)
+async def test_filter_cookie_with_unicode_domain(loop) -> None:
+    jar = CookieJar()
     jar.update_cookies(SimpleCookie(
         "idna-domain-first=first; Domain=xn--9caa.com; Path=/; "
     ))
@@ -189,13 +228,7 @@ def test_filter_cookie_with_unicode_domain(loop):
     assert len(jar.filter_cookies(URL("http://xn--9caa.com"))) == 1
 
 
-def test_ctor_ith_default_loop(loop):
-    asyncio.set_event_loop(loop)
-    jar = CookieJar()
-    assert jar._loop is loop
-
-
-def test_domain_filter_ip_cookie_send(loop):
+async def test_domain_filter_ip_cookie_send(loop) -> None:
     jar = CookieJar(loop=loop)
     cookies = SimpleCookie(
         "shared-cookie=first; "
@@ -226,14 +259,14 @@ def test_domain_filter_ip_cookie_send(loop):
     assert cookies_sent == 'Cookie: shared-cookie=first'
 
 
-def test_domain_filter_ip_cookie_receive(loop, cookies_to_receive):
-    jar = CookieJar(loop=loop)
+async def test_domain_filter_ip_cookie_receive(cookies_to_receive) -> None:
+    jar = CookieJar()
 
     jar.update_cookies(cookies_to_receive, URL("http://1.2.3.4/"))
     assert len(jar) == 0
 
 
-def test_preserving_ip_domain_cookies(loop):
+async def test_preserving_ip_domain_cookies(loop) -> None:
     jar = CookieJar(loop=loop, unsafe=True)
     jar.update_cookies(SimpleCookie(
         "shared-cookie=first; "
@@ -245,7 +278,7 @@ def test_preserving_ip_domain_cookies(loop):
                             'Cookie: shared-cookie=first')
 
 
-def test_preserving_quoted_cookies(loop):
+async def test_preserving_quoted_cookies(loop) -> None:
     jar = CookieJar(loop=loop, unsafe=True)
     jar.update_cookies(SimpleCookie(
         "ip-cookie=\"second\"; Domain=127.0.0.1;"
@@ -255,7 +288,7 @@ def test_preserving_quoted_cookies(loop):
     assert cookies_sent == 'Cookie: ip-cookie=\"second\"'
 
 
-def test_ignore_domain_ending_with_dot(loop):
+async def test_ignore_domain_ending_with_dot(loop) -> None:
     jar = CookieJar(loop=loop, unsafe=True)
     jar.update_cookies(SimpleCookie("cookie=val; Domain=example.com.;"),
                        URL("http://www.example.com"))
@@ -272,7 +305,9 @@ class TestCookieJarBase(unittest.TestCase):
         asyncio.set_event_loop(None)
 
         # N.B. those need to be overridden in child test cases
-        self.jar = CookieJar(loop=self.loop)
+        async def make_jar():
+            return CookieJar()
+        self.jar = self.loop.run_until_complete(make_jar())
 
     def tearDown(self):
         self.loop.close()
@@ -333,20 +368,31 @@ class TestCookieJarSafe(TestCookieJarBase):
             "wrong-path-cookie=nineth; Domain=pathtest.com; Path=somepath;"
         )
 
-        self.jar = CookieJar(loop=self.loop)
+        async def make_jar():
+            return CookieJar()
+        self.jar = self.loop.run_until_complete(make_jar())
 
     def timed_request(self, url, update_time, send_time):
-        with mock.patch.object(self.loop, 'time', return_value=update_time):
+        if isinstance(update_time, int):
+            update_time = datetime.timedelta(seconds=update_time)
+        elif isinstance(update_time, float):
+            update_time = datetime.datetime.fromtimestamp(update_time)
+        if isinstance(send_time, int):
+            send_time = datetime.timedelta(seconds=send_time)
+        elif isinstance(send_time, float):
+            send_time = datetime.datetime.fromtimestamp(send_time)
+
+        with freeze_time(update_time):
             self.jar.update_cookies(self.cookies_to_send)
 
-        with mock.patch.object(self.loop, 'time', return_value=send_time):
+        with freeze_time(send_time):
             cookies_sent = self.jar.filter_cookies(URL(url))
 
         self.jar.clear()
 
         return cookies_sent
 
-    def test_domain_filter_same_host(self):
+    def test_domain_filter_same_host(self) -> None:
         cookies_sent, cookies_received = (
             self.request_reply_with_same_url("http://example.com/"))
 
@@ -362,7 +408,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "dotted-domain-cookie"
         })
 
-    def test_domain_filter_same_host_and_subdomain(self):
+    def test_domain_filter_same_host_and_subdomain(self) -> None:
         cookies_sent, cookies_received = (
             self.request_reply_with_same_url("http://test1.example.com/"))
 
@@ -380,7 +426,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "dotted-domain-cookie"
         })
 
-    def test_domain_filter_same_host_diff_subdomain(self):
+    def test_domain_filter_same_host_diff_subdomain(self) -> None:
         cookies_sent, cookies_received = (
             self.request_reply_with_same_url("http://different.example.com/"))
 
@@ -396,7 +442,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "dotted-domain-cookie"
         })
 
-    def test_domain_filter_diff_host(self):
+    def test_domain_filter_diff_host(self) -> None:
         cookies_sent, cookies_received = (
             self.request_reply_with_same_url("http://different.org/"))
 
@@ -410,7 +456,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "different-domain-cookie"
         })
 
-    def test_domain_filter_host_only(self):
+    def test_domain_filter_host_only(self) -> None:
         self.jar.update_cookies(self.cookies_to_receive,
                                 URL("http://example.com/"))
 
@@ -420,7 +466,7 @@ class TestCookieJarSafe(TestCookieJarBase):
         cookies_sent = self.jar.filter_cookies(URL("http://different.org/"))
         self.assertNotIn("unconstrained-cookie", set(cookies_sent.keys()))
 
-    def test_secure_filter(self):
+    def test_secure_filter(self) -> None:
         cookies_sent, _ = (
             self.request_reply_with_same_url("http://secure.com/"))
 
@@ -436,7 +482,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "secure-cookie"
         })
 
-    def test_path_filter_root(self):
+    def test_path_filter_root(self) -> None:
         cookies_sent, _ = (
             self.request_reply_with_same_url("http://pathtest.com/"))
 
@@ -446,7 +492,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "path1-cookie"
         })
 
-    def test_path_filter_folder(self):
+    def test_path_filter_folder(self) -> None:
 
         cookies_sent, _ = (
             self.request_reply_with_same_url("http://pathtest.com/one/"))
@@ -458,7 +504,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "path2-cookie"
         })
 
-    def test_path_filter_file(self):
+    def test_path_filter_file(self) -> None:
 
         cookies_sent, _ = self.request_reply_with_same_url(
             "http://pathtest.com/one/two")
@@ -471,7 +517,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "path3-cookie"
         })
 
-    def test_path_filter_subfolder(self):
+    def test_path_filter_subfolder(self) -> None:
 
         cookies_sent, _ = self.request_reply_with_same_url(
             "http://pathtest.com/one/two/")
@@ -485,7 +531,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "path4-cookie"
         })
 
-    def test_path_filter_subsubfolder(self):
+    def test_path_filter_subsubfolder(self) -> None:
 
         cookies_sent, _ = self.request_reply_with_same_url(
             "http://pathtest.com/one/two/three/")
@@ -499,7 +545,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "path4-cookie"
         })
 
-    def test_path_filter_different_folder(self):
+    def test_path_filter_different_folder(self) -> None:
 
         cookies_sent, _ = (
             self.request_reply_with_same_url("http://pathtest.com/hundred/"))
@@ -510,7 +556,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "path1-cookie"
         })
 
-    def test_path_value(self):
+    def test_path_value(self) -> None:
         _, cookies_received = (
             self.request_reply_with_same_url("http://pathtest.com/"))
 
@@ -525,7 +571,7 @@ class TestCookieJarSafe(TestCookieJarBase):
         self.assertEqual(cookies_received["path-cookie"]["path"], "/somepath")
         self.assertEqual(cookies_received["wrong-path-cookie"]["path"], "/")
 
-    def test_expires(self):
+    def test_expires(self) -> None:
         ts_before = datetime.datetime(
             1975, 1, 1, tzinfo=datetime.timezone.utc).timestamp()
 
@@ -547,7 +593,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "shared-cookie"
         })
 
-    def test_max_age(self):
+    def test_max_age(self) -> None:
         cookies_sent = self.timed_request(
             "http://maxagetest.com/", 1000, 1000)
 
@@ -563,7 +609,7 @@ class TestCookieJarSafe(TestCookieJarBase):
             "shared-cookie"
         })
 
-    def test_invalid_values(self):
+    def test_invalid_values(self) -> None:
         cookies_sent, cookies_received = (
             self.request_reply_with_same_url("http://invalid-values.com/"))
 
@@ -579,7 +625,7 @@ class TestCookieJarSafe(TestCookieJarBase):
         cookie = cookies_sent["invalid-expires-cookie"]
         self.assertEqual(cookie["expires"], "")
 
-    def test_cookie_not_expired_when_added_after_removal(self):
+    def test_cookie_not_expired_when_added_after_removal(self) -> None:
         """Test case for https://github.com/aio-libs/aiohttp/issues/2084"""
         timestamps = [533588.993, 533588.993, 533588.993,
                       533588.993, 533589.093, 533589.093]
@@ -588,7 +634,9 @@ class TestCookieJarSafe(TestCookieJarBase):
         loop.time.side_effect = itertools.chain(
             timestamps, itertools.cycle([timestamps[-1]]))
 
-        jar = CookieJar(unsafe=True, loop=loop)
+        async def make_jar():
+            return CookieJar(unsafe=True)
+        jar = self.loop.run_until_complete(make_jar())
         # Remove `foo` cookie.
         jar.update_cookies(SimpleCookie('foo=""; Max-Age=0'))
         # Set `foo` cookie to `bar`.
@@ -598,13 +646,13 @@ class TestCookieJarSafe(TestCookieJarBase):
         assert len(jar) == 1
 
 
-def test_dummy_cookie_jar(loop):
+async def test_dummy_cookie_jar() -> None:
     cookie = SimpleCookie('foo=bar; Domain=example.com;')
-    dummy_jar = DummyCookieJar(loop=loop)
+    dummy_jar = DummyCookieJar()
     assert len(dummy_jar) == 0
     dummy_jar.update_cookies(cookie)
     assert len(dummy_jar) == 0
     with pytest.raises(StopIteration):
         next(iter(dummy_jar))
-    assert dummy_jar.filter_cookies(URL("http://example.com/")) is None
+    assert not dummy_jar.filter_cookies(URL("http://example.com/"))
     dummy_jar.clear()
