@@ -1,24 +1,28 @@
-import asyncio
 from types import SimpleNamespace
 from unittest.mock import Mock
 
 import pytest
 
-from aiohttp.tracing import (Trace, TraceConfig,
-                             TraceConnectionCreateEndParams,
-                             TraceConnectionCreateStartParams,
-                             TraceConnectionQueuedEndParams,
-                             TraceConnectionQueuedStartParams,
-                             TraceConnectionReuseconnParams,
-                             TraceDnsCacheHitParams, TraceDnsCacheMissParams,
-                             TraceDnsResolveHostEndParams,
-                             TraceDnsResolveHostStartParams,
-                             TraceRequestChunkSentParams,
-                             TraceRequestEndParams,
-                             TraceRequestExceptionParams,
-                             TraceRequestRedirectParams,
-                             TraceRequestStartParams,
-                             TraceResponseChunkReceivedParams)
+from aiohttp.test_utils import make_mocked_coro
+from aiohttp.tracing import (
+    Trace,
+    TraceConfig,
+    TraceConnectionCreateEndParams,
+    TraceConnectionCreateStartParams,
+    TraceConnectionQueuedEndParams,
+    TraceConnectionQueuedStartParams,
+    TraceConnectionReuseconnParams,
+    TraceDnsCacheHitParams,
+    TraceDnsCacheMissParams,
+    TraceDnsResolveHostEndParams,
+    TraceDnsResolveHostStartParams,
+    TraceRequestChunkSentParams,
+    TraceRequestEndParams,
+    TraceRequestExceptionParams,
+    TraceRequestRedirectParams,
+    TraceRequestStartParams,
+    TraceResponseChunkReceivedParams,
+)
 
 
 class TestTraceConfig:
@@ -141,7 +145,7 @@ class TestTrace:
     async def test_send(self, signal, params, param_obj) -> None:
         session = Mock()
         trace_request_ctx = Mock()
-        callback = Mock(side_effect=asyncio.coroutine(Mock()))
+        callback = Mock(side_effect=make_mocked_coro(Mock()))
 
         trace_config = TraceConfig()
         getattr(trace_config, "on_%s" % signal).append(callback)
