@@ -8,18 +8,24 @@ from unittest import mock
 
 import pytest
 from async_generator import async_generator, yield_
-from multidict import MultiDict
+from multidict import CIMultiDictProxy, MultiDict
 from yarl import URL
 
 import aiohttp
-from aiohttp import (FormData, HttpVersion10, HttpVersion11, TraceConfig,
-                     multipart, web)
-
+from aiohttp import (
+    FormData,
+    HttpVersion10,
+    HttpVersion11,
+    TraceConfig,
+    multipart,
+    web,
+)
+from aiohttp.test_utils import make_mocked_coro
 
 try:
     import ssl
 except ImportError:
-    ssl = False
+    ssl = None  # type: ignore
 
 
 @pytest.fixture
@@ -29,10 +35,10 @@ def here():
 
 @pytest.fixture
 def fname(here):
-    return here / 'sample.key'
+    return here / 'conftest.py'
 
 
-async def test_simple_get(aiohttp_client):
+async def test_simple_get(aiohttp_client) -> None:
 
     async def handler(request):
         body = await request.read()
@@ -49,7 +55,7 @@ async def test_simple_get(aiohttp_client):
     assert 'OK' == txt
 
 
-async def test_simple_get_with_text(aiohttp_client):
+async def test_simple_get_with_text(aiohttp_client) -> None:
 
     async def handler(request):
         body = await request.read()
@@ -66,7 +72,9 @@ async def test_simple_get_with_text(aiohttp_client):
     assert 'OK' == txt
 
 
-async def test_handler_returns_not_response(aiohttp_server, aiohttp_client):
+async def test_handler_returns_not_response(aiohttp_server,
+                                            aiohttp_client) -> None:
+    asyncio.get_event_loop().set_debug(True)
     logger = mock.Mock()
 
     async def handler(request):
@@ -77,13 +85,35 @@ async def test_handler_returns_not_response(aiohttp_server, aiohttp_client):
     server = await aiohttp_server(app, logger=logger)
     client = await aiohttp_client(server)
 
-    resp = await client.get('/')
-    assert 500 == resp.status
+    with pytest.raises(aiohttp.ServerDisconnectedError):
+        await client.get('/')
 
-    assert logger.exception.called
+    logger.exception.assert_called_with('Unhandled runtime exception',
+                                        exc_info=mock.ANY)
 
 
-async def test_head_returns_empty_body(aiohttp_client):
+async def test_handler_returns_none(aiohttp_server,
+                                    aiohttp_client) -> None:
+    asyncio.get_event_loop().set_debug(True)
+    logger = mock.Mock()
+
+    async def handler(request):
+        return None
+
+    app = web.Application()
+    app.router.add_get('/', handler)
+    server = await aiohttp_server(app, logger=logger)
+    client = await aiohttp_client(server)
+
+    with pytest.raises(aiohttp.ServerDisconnectedError):
+        await client.get('/')
+
+    # Actual error text is placed in exc_info
+    logger.exception.assert_called_with('Unhandled runtime exception',
+                                        exc_info=mock.ANY)
+
+
+async def test_head_returns_empty_body(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response(body=b'test')
@@ -98,7 +128,7 @@ async def test_head_returns_empty_body(aiohttp_client):
     assert '' == txt
 
 
-async def test_response_before_complete(aiohttp_client):
+async def test_response_before_complete(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response(body=b'OK')
@@ -115,7 +145,7 @@ async def test_response_before_complete(aiohttp_client):
     assert 'OK' == text
 
 
-async def test_post_form(aiohttp_client):
+async def test_post_form(aiohttp_client) -> None:
 
     async def handler(request):
         data = await request.post()
@@ -132,7 +162,7 @@ async def test_post_form(aiohttp_client):
     assert 'OK' == txt
 
 
-async def test_post_text(aiohttp_client):
+async def test_post_text(aiohttp_client) -> None:
 
     async def handler(request):
         data = await request.text()
@@ -151,7 +181,7 @@ async def test_post_text(aiohttp_client):
     assert 'русский' == txt
 
 
-async def test_post_json(aiohttp_client):
+async def test_post_json(aiohttp_client) -> None:
 
     dct = {'key': 'текст'}
 
@@ -176,7 +206,7 @@ async def test_post_json(aiohttp_client):
     assert dct == data
 
 
-async def test_multipart(aiohttp_client):
+async def test_multipart(aiohttp_client) -> None:
     with multipart.MultipartWriter() as writer:
         writer.append('test')
         writer.append_json({'passed': True})
@@ -205,12 +235,32 @@ async def test_multipart(aiohttp_client):
     app.router.add_post('/', handler)
     client = await aiohttp_client(app)
 
-    resp = await client.post('/', data=writer, headers=writer.headers)
+    resp = await client.post('/', data=writer)
     assert 200 == resp.status
     await resp.release()
 
 
-async def test_multipart_content_transfer_encoding(aiohttp_client):
+async def test_multipart_empty(aiohttp_client) -> None:
+    with multipart.MultipartWriter() as writer:
+        pass
+
+    async def handler(request):
+        reader = await request.multipart()
+        assert isinstance(reader, multipart.MultipartReader)
+        async for part in reader:
+            assert False, 'Unexpected part found in reader: {!r}'.format(part)
+        return web.Response()
+
+    app = web.Application()
+    app.router.add_post('/', handler)
+    client = await aiohttp_client(app)
+
+    resp = await client.post('/', data=writer)
+    assert 200 == resp.status
+    await resp.release()
+
+
+async def test_multipart_content_transfer_encoding(aiohttp_client) -> None:
     """For issue #1168"""
     with multipart.MultipartWriter() as writer:
         writer.append(b'\x00' * 10,
@@ -235,12 +285,12 @@ async def test_multipart_content_transfer_encoding(aiohttp_client):
     app.router.add_post('/', handler)
     client = await aiohttp_client(app)
 
-    resp = await client.post('/', data=writer, headers=writer.headers)
+    resp = await client.post('/', data=writer)
     assert 200 == resp.status
     await resp.release()
 
 
-async def test_render_redirect(aiohttp_client):
+async def test_render_redirect(aiohttp_client) -> None:
 
     async def handler(request):
         raise web.HTTPMovedPermanently(location='/path')
@@ -256,7 +306,7 @@ async def test_render_redirect(aiohttp_client):
     assert '/path' == resp.headers['location']
 
 
-async def test_post_single_file(aiohttp_client):
+async def test_post_single_file(aiohttp_client) -> None:
 
     here = pathlib.Path(__file__).parent
 
@@ -269,7 +319,7 @@ async def test_post_single_file(aiohttp_client):
 
     async def handler(request):
         data = await request.post()
-        assert ['sample.crt'] == list(data.keys())
+        assert ['data.unknown_mime_type'] == list(data.keys())
         for fs in data.values():
             check_file(fs)
             fs.file.close()
@@ -280,13 +330,13 @@ async def test_post_single_file(aiohttp_client):
     app.router.add_post('/', handler)
     client = await aiohttp_client(app)
 
-    fname = here / 'sample.crt'
+    fname = here / 'data.unknown_mime_type'
 
     resp = await client.post('/', data=[fname.open()])
     assert 200 == resp.status
 
 
-async def test_files_upload_with_same_key(aiohttp_client):
+async def test_files_upload_with_same_key(aiohttp_client) -> None:
 
     async def handler(request):
         data = await request.post()
@@ -319,7 +369,7 @@ async def test_files_upload_with_same_key(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_post_files(aiohttp_client):
+async def test_post_files(aiohttp_client) -> None:
 
     here = pathlib.Path(__file__).parent
 
@@ -332,7 +382,7 @@ async def test_post_files(aiohttp_client):
 
     async def handler(request):
         data = await request.post()
-        assert ['sample.crt', 'sample.key'] == list(data.keys())
+        assert ['data.unknown_mime_type', 'conftest.py'] == list(data.keys())
         for fs in data.values():
             check_file(fs)
             fs.file.close()
@@ -343,13 +393,13 @@ async def test_post_files(aiohttp_client):
     app.router.add_post('/', handler)
     client = await aiohttp_client(app)
 
-    with (here / 'sample.crt').open() as f1:
-        with (here / 'sample.key').open() as f2:
+    with (here / 'data.unknown_mime_type').open() as f1:
+        with (here / 'conftest.py').open() as f2:
             resp = await client.post('/', data=[f1, f2])
             assert 200 == resp.status
 
 
-async def test_release_post_data(aiohttp_client):
+async def test_release_post_data(aiohttp_client) -> None:
 
     async def handler(request):
         await request.release()
@@ -365,7 +415,8 @@ async def test_release_post_data(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_POST_DATA_with_content_transfer_encoding(aiohttp_client):
+async def test_POST_DATA_with_content_transfer_encoding(
+        aiohttp_client) -> None:
 
     async def handler(request):
         data = await request.post()
@@ -384,7 +435,7 @@ async def test_POST_DATA_with_content_transfer_encoding(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_post_form_with_duplicate_keys(aiohttp_client):
+async def test_post_form_with_duplicate_keys(aiohttp_client) -> None:
 
     async def handler(request):
         data = await request.post()
@@ -400,12 +451,12 @@ async def test_post_form_with_duplicate_keys(aiohttp_client):
     assert 200 == resp.status
 
 
-def test_repr_for_application():
+def test_repr_for_application() -> None:
     app = web.Application()
     assert "<Application 0x{:x}>".format(id(app)) == repr(app)
 
 
-async def test_expect_default_handler_unknown(aiohttp_client):
+async def test_expect_default_handler_unknown(aiohttp_client) -> None:
     """Test default Expect handler for unknown Expect value.
 
     A server that does not understand or is unable to comply with any of
@@ -430,7 +481,7 @@ async def test_expect_default_handler_unknown(aiohttp_client):
     assert 417 == resp.status
 
 
-async def test_100_continue(aiohttp_client):
+async def test_100_continue(aiohttp_client) -> None:
 
     async def handler(request):
         data = await request.post()
@@ -449,7 +500,7 @@ async def test_100_continue(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_100_continue_custom(aiohttp_client):
+async def test_100_continue_custom(aiohttp_client) -> None:
 
     expect_received = False
 
@@ -477,7 +528,7 @@ async def test_100_continue_custom(aiohttp_client):
     assert expect_received
 
 
-async def test_100_continue_custom_response(aiohttp_client):
+async def test_100_continue_custom_response(aiohttp_client) -> None:
 
     async def handler(request):
         data = await request.post()
@@ -508,7 +559,7 @@ async def test_100_continue_custom_response(aiohttp_client):
     assert 403 == resp.status
 
 
-async def test_100_continue_for_not_found(aiohttp_client):
+async def test_100_continue_for_not_found(aiohttp_client) -> None:
 
     app = web.Application()
     client = await aiohttp_client(app)
@@ -517,7 +568,7 @@ async def test_100_continue_for_not_found(aiohttp_client):
     assert 404 == resp.status
 
 
-async def test_100_continue_for_not_allowed(aiohttp_client):
+async def test_100_continue_for_not_allowed(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response()
@@ -530,7 +581,7 @@ async def test_100_continue_for_not_allowed(aiohttp_client):
     assert 405 == resp.status
 
 
-async def test_http11_keep_alive_default(aiohttp_client):
+async def test_http11_keep_alive_default(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response()
@@ -546,7 +597,7 @@ async def test_http11_keep_alive_default(aiohttp_client):
 
 
 @pytest.mark.xfail
-async def test_http10_keep_alive_default(aiohttp_client):
+async def test_http10_keep_alive_default(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response()
@@ -561,7 +612,7 @@ async def test_http10_keep_alive_default(aiohttp_client):
     assert resp.headers['Connection'] == 'keep-alive'
 
 
-async def test_http10_keep_alive_with_headers_close(aiohttp_client):
+async def test_http10_keep_alive_with_headers_close(aiohttp_client) -> None:
 
     async def handler(request):
         await request.read()
@@ -578,7 +629,7 @@ async def test_http10_keep_alive_with_headers_close(aiohttp_client):
     assert 'Connection' not in resp.headers
 
 
-async def test_http10_keep_alive_with_headers(aiohttp_client):
+async def test_http10_keep_alive_with_headers(aiohttp_client) -> None:
 
     async def handler(request):
         await request.read()
@@ -595,7 +646,7 @@ async def test_http10_keep_alive_with_headers(aiohttp_client):
     assert resp.headers['Connection'] == 'keep-alive'
 
 
-async def test_upload_file(aiohttp_client):
+async def test_upload_file(aiohttp_client) -> None:
 
     here = pathlib.Path(__file__).parent
     fname = here / 'aiohttp.png'
@@ -616,7 +667,7 @@ async def test_upload_file(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_upload_file_object(aiohttp_client):
+async def test_upload_file_object(aiohttp_client) -> None:
     here = pathlib.Path(__file__).parent
     fname = here / 'aiohttp.png'
     with fname.open('rb') as f:
@@ -637,7 +688,7 @@ async def test_upload_file_object(aiohttp_client):
         assert 200 == resp.status
 
 
-async def test_empty_content_for_query_without_body(aiohttp_client):
+async def test_empty_content_for_query_without_body(aiohttp_client) -> None:
 
     async def handler(request):
         assert not request.body_exists
@@ -654,7 +705,7 @@ async def test_empty_content_for_query_without_body(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_empty_content_for_query_with_body(aiohttp_client):
+async def test_empty_content_for_query_with_body(aiohttp_client) -> None:
 
     async def handler(request):
         assert request.body_exists
@@ -672,7 +723,7 @@ async def test_empty_content_for_query_with_body(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_get_with_empty_arg(aiohttp_client):
+async def test_get_with_empty_arg(aiohttp_client) -> None:
 
     async def handler(request):
         assert 'arg' in request.query
@@ -687,7 +738,7 @@ async def test_get_with_empty_arg(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_large_header(aiohttp_client):
+async def test_large_header(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response()
@@ -701,7 +752,7 @@ async def test_large_header(aiohttp_client):
     assert 400 == resp.status
 
 
-async def test_large_header_allowed(aiohttp_client, aiohttp_server):
+async def test_large_header_allowed(aiohttp_client, aiohttp_server) -> None:
 
     async def handler(request):
         return web.Response()
@@ -716,7 +767,7 @@ async def test_large_header_allowed(aiohttp_client, aiohttp_server):
     assert 200 == resp.status
 
 
-async def test_get_with_empty_arg_with_equal(aiohttp_client):
+async def test_get_with_empty_arg_with_equal(aiohttp_client) -> None:
 
     async def handler(request):
         assert 'arg' in request.query
@@ -731,7 +782,7 @@ async def test_get_with_empty_arg_with_equal(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_response_with_async_gen(aiohttp_client, fname):
+async def test_response_with_async_gen(aiohttp_client, fname) -> None:
 
     with fname.open('rb') as f:
         data = f.read()
@@ -761,7 +812,7 @@ async def test_response_with_async_gen(aiohttp_client, fname):
     assert resp.headers.get('Content-Length') == str(len(resp_data))
 
 
-async def test_response_with_streamer(aiohttp_client, fname):
+async def test_response_with_streamer(aiohttp_client, fname) -> None:
 
     with fname.open('rb') as f:
         data = f.read()
@@ -792,7 +843,8 @@ async def test_response_with_streamer(aiohttp_client, fname):
     assert resp.headers.get('Content-Length') == str(len(resp_data))
 
 
-async def test_response_with_async_gen_no_params(aiohttp_client, fname):
+async def test_response_with_async_gen_no_params(aiohttp_client,
+                                                 fname) -> None:
 
     with fname.open('rb') as f:
         data = f.read()
@@ -822,7 +874,7 @@ async def test_response_with_async_gen_no_params(aiohttp_client, fname):
     assert resp.headers.get('Content-Length') == str(len(resp_data))
 
 
-async def test_response_with_streamer_no_params(aiohttp_client, fname):
+async def test_response_with_streamer_no_params(aiohttp_client, fname) -> None:
 
     with fname.open('rb') as f:
         data = f.read()
@@ -853,7 +905,7 @@ async def test_response_with_streamer_no_params(aiohttp_client, fname):
     assert resp.headers.get('Content-Length') == str(len(resp_data))
 
 
-async def test_response_with_file(aiohttp_client, fname):
+async def test_response_with_file(aiohttp_client, fname) -> None:
 
     with fname.open('rb') as f:
         data = f.read()
@@ -868,15 +920,20 @@ async def test_response_with_file(aiohttp_client, fname):
     resp = await client.get('/')
     assert 200 == resp.status
     resp_data = await resp.read()
+    expected_content_disposition = (
+        'attachment; filename="conftest.py"; filename*=utf-8\'\'conftest.py'
+    )
     assert resp_data == data
     assert resp.headers.get('Content-Type') in (
-        'application/octet-stream', 'application/pgp-keys')
+        'application/octet-stream', 'text/x-python', 'text/plain',
+    )
     assert resp.headers.get('Content-Length') == str(len(resp_data))
-    assert (resp.headers.get('Content-Disposition') ==
-            'attachment; filename="sample.key"; filename*=utf-8\'\'sample.key')
+    assert (
+        resp.headers.get('Content-Disposition') == expected_content_disposition
+    )
 
 
-async def test_response_with_file_ctype(aiohttp_client, fname):
+async def test_response_with_file_ctype(aiohttp_client, fname) -> None:
 
     with fname.open('rb') as f:
         data = f.read()
@@ -892,14 +949,18 @@ async def test_response_with_file_ctype(aiohttp_client, fname):
     resp = await client.get('/')
     assert 200 == resp.status
     resp_data = await resp.read()
+    expected_content_disposition = (
+        'attachment; filename="conftest.py"; filename*=utf-8\'\'conftest.py'
+    )
     assert resp_data == data
     assert resp.headers.get('Content-Type') == 'text/binary'
     assert resp.headers.get('Content-Length') == str(len(resp_data))
-    assert (resp.headers.get('Content-Disposition') ==
-            'attachment; filename="sample.key"; filename*=utf-8\'\'sample.key')
+    assert (
+        resp.headers.get('Content-Disposition') == expected_content_disposition
+    )
 
 
-async def test_response_with_payload_disp(aiohttp_client, fname):
+async def test_response_with_payload_disp(aiohttp_client, fname) -> None:
 
     with fname.open('rb') as f:
         data = f.read()
@@ -924,7 +985,7 @@ async def test_response_with_payload_disp(aiohttp_client, fname):
             'inline; filename="test.txt"; filename*=utf-8\'\'test.txt')
 
 
-async def test_response_with_payload_stringio(aiohttp_client, fname):
+async def test_response_with_payload_stringio(aiohttp_client, fname) -> None:
 
     async def handler(request):
         return web.Response(body=io.StringIO('test'))
@@ -939,7 +1000,7 @@ async def test_response_with_payload_stringio(aiohttp_client, fname):
     assert resp_data == b'test'
 
 
-async def test_response_with_precompressed_body_gzip(aiohttp_client):
+async def test_response_with_precompressed_body_gzip(aiohttp_client) -> None:
 
     async def handler(request):
         headers = {'Content-Encoding': 'gzip'}
@@ -958,7 +1019,8 @@ async def test_response_with_precompressed_body_gzip(aiohttp_client):
     assert resp.headers.get('Content-Encoding') == 'gzip'
 
 
-async def test_response_with_precompressed_body_deflate(aiohttp_client):
+async def test_response_with_precompressed_body_deflate(
+        aiohttp_client) -> None:
 
     async def handler(request):
         headers = {'Content-Encoding': 'deflate'}
@@ -977,7 +1039,7 @@ async def test_response_with_precompressed_body_deflate(aiohttp_client):
     assert resp.headers.get('Content-Encoding') == 'deflate'
 
 
-async def test_bad_request_payload(aiohttp_client):
+async def test_bad_request_payload(aiohttp_client) -> None:
 
     async def handler(request):
         assert request.method == 'POST'
@@ -996,7 +1058,7 @@ async def test_bad_request_payload(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_stream_response_multiple_chunks(aiohttp_client):
+async def test_stream_response_multiple_chunks(aiohttp_client) -> None:
 
     async def handler(request):
         resp = web.StreamResponse()
@@ -1017,7 +1079,7 @@ async def test_stream_response_multiple_chunks(aiohttp_client):
     assert b'xyz' == data
 
 
-async def test_start_without_routes(aiohttp_client):
+async def test_start_without_routes(aiohttp_client) -> None:
 
     app = web.Application()
     client = await aiohttp_client(app)
@@ -1026,7 +1088,7 @@ async def test_start_without_routes(aiohttp_client):
     assert 404 == resp.status
 
 
-async def test_requests_count(aiohttp_client):
+async def test_requests_count(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response()
@@ -1049,7 +1111,7 @@ async def test_requests_count(aiohttp_client):
     assert client.server.handler.requests_count == 3
 
 
-async def test_redirect_url(aiohttp_client):
+async def test_redirect_url(aiohttp_client) -> None:
 
     async def redirector(request):
         raise web.HTTPFound(location=URL('/redirected'))
@@ -1066,7 +1128,7 @@ async def test_redirect_url(aiohttp_client):
     assert resp.status == 200
 
 
-async def test_simple_subapp(aiohttp_client):
+async def test_simple_subapp(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response(text="OK")
@@ -1083,7 +1145,7 @@ async def test_simple_subapp(aiohttp_client):
     assert 'OK' == txt
 
 
-async def test_subapp_reverse_url(aiohttp_client):
+async def test_subapp_reverse_url(aiohttp_client) -> None:
 
     async def handler(request):
         raise web.HTTPMovedPermanently(
@@ -1106,7 +1168,7 @@ async def test_subapp_reverse_url(aiohttp_client):
     assert resp.url.path == '/path/final'
 
 
-async def test_subapp_reverse_variable_url(aiohttp_client):
+async def test_subapp_reverse_variable_url(aiohttp_client) -> None:
 
     async def handler(request):
         raise web.HTTPMovedPermanently(
@@ -1129,7 +1191,7 @@ async def test_subapp_reverse_variable_url(aiohttp_client):
     assert resp.url.path == '/path/final'
 
 
-async def test_subapp_reverse_static_url(aiohttp_client):
+async def test_subapp_reverse_static_url(aiohttp_client) -> None:
     fname = 'aiohttp.png'
 
     async def handler(request):
@@ -1152,7 +1214,7 @@ async def test_subapp_reverse_static_url(aiohttp_client):
         assert body == f.read()
 
 
-async def test_subapp_app(aiohttp_client):
+async def test_subapp_app(aiohttp_client) -> None:
 
     async def handler(request):
         assert request.app is subapp
@@ -1170,7 +1232,7 @@ async def test_subapp_app(aiohttp_client):
     assert 'OK' == txt
 
 
-async def test_subapp_not_found(aiohttp_client):
+async def test_subapp_not_found(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response(text='OK')
@@ -1185,7 +1247,7 @@ async def test_subapp_not_found(aiohttp_client):
     assert resp.status == 404
 
 
-async def test_subapp_not_found2(aiohttp_client):
+async def test_subapp_not_found2(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response(text='OK')
@@ -1200,7 +1262,7 @@ async def test_subapp_not_found2(aiohttp_client):
     assert resp.status == 404
 
 
-async def test_subapp_not_allowed(aiohttp_client):
+async def test_subapp_not_allowed(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response(text='OK')
@@ -1216,7 +1278,7 @@ async def test_subapp_not_allowed(aiohttp_client):
     assert resp.headers['Allow'] == 'GET,HEAD'
 
 
-async def test_subapp_cannot_add_app_in_handler(aiohttp_client):
+async def test_subapp_cannot_add_app_in_handler(aiohttp_client) -> None:
 
     async def handler(request):
         request.match_info.add_app(app)
@@ -1232,7 +1294,7 @@ async def test_subapp_cannot_add_app_in_handler(aiohttp_client):
     assert resp.status == 500
 
 
-async def test_subapp_middlewares(aiohttp_client):
+async def test_subapp_middlewares(aiohttp_client) -> None:
     order = []
 
     async def handler(request):
@@ -1263,7 +1325,7 @@ async def test_subapp_middlewares(aiohttp_client):
             (2, subapp2), (2, subapp1), (2, app)] == order
 
 
-async def test_subapp_on_response_prepare(aiohttp_client):
+async def test_subapp_on_response_prepare(aiohttp_client) -> None:
     order = []
 
     async def handler(request):
@@ -1292,7 +1354,7 @@ async def test_subapp_on_response_prepare(aiohttp_client):
     assert [app, subapp1, subapp2] == order
 
 
-async def test_subapp_on_startup(aiohttp_server):
+async def test_subapp_on_startup(aiohttp_server) -> None:
     order = []
 
     async def on_signal(app):
@@ -1312,7 +1374,7 @@ async def test_subapp_on_startup(aiohttp_server):
     assert [app, subapp1, subapp2] == order
 
 
-async def test_subapp_on_shutdown(aiohttp_server):
+async def test_subapp_on_shutdown(aiohttp_server) -> None:
     order = []
 
     async def on_signal(app):
@@ -1333,7 +1395,7 @@ async def test_subapp_on_shutdown(aiohttp_server):
     assert [app, subapp1, subapp2] == order
 
 
-async def test_subapp_on_cleanup(aiohttp_server):
+async def test_subapp_on_cleanup(aiohttp_server) -> None:
     order = []
 
     async def on_signal(app):
@@ -1403,7 +1465,7 @@ async def test_subapp_middleware_context(aiohttp_client,
     assert expected == values
 
 
-async def test_custom_date_header(aiohttp_client):
+async def test_custom_date_header(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response(headers={'Date': 'Sun, 30 Oct 2016 03:13:52 GMT'})
@@ -1417,7 +1479,7 @@ async def test_custom_date_header(aiohttp_client):
     assert resp.headers['Date'] == 'Sun, 30 Oct 2016 03:13:52 GMT'
 
 
-async def test_response_prepared_with_clone(aiohttp_client):
+async def test_response_prepared_with_clone(aiohttp_client) -> None:
 
     async def handler(request):
         cloned = request.clone()
@@ -1433,7 +1495,7 @@ async def test_response_prepared_with_clone(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_app_max_client_size(aiohttp_client):
+async def test_app_max_client_size(aiohttp_client) -> None:
 
     async def handler(request):
         await request.post()
@@ -1448,10 +1510,11 @@ async def test_app_max_client_size(aiohttp_client):
         resp = await client.post('/', data=data)
     assert 413 == resp.status
     resp_text = await resp.text()
-    assert 'Request Entity Too Large' in resp_text
+    assert 'Maximum request body size 1048576 exceeded, ' \
+           'actual body size 1048591' in resp_text
 
 
-async def test_app_max_client_size_adjusted(aiohttp_client):
+async def test_app_max_client_size_adjusted(aiohttp_client) -> None:
 
     async def handler(request):
         await request.post()
@@ -1473,10 +1536,11 @@ async def test_app_max_client_size_adjusted(aiohttp_client):
         resp = await client.post('/', data=too_large_data)
     assert 413 == resp.status
     resp_text = await resp.text()
-    assert 'Request Entity Too Large' in resp_text
+    assert 'Maximum request body size 2097152 exceeded, ' \
+           'actual body size 2097166' in resp_text
 
 
-async def test_app_max_client_size_none(aiohttp_client):
+async def test_app_max_client_size_none(aiohttp_client) -> None:
 
     async def handler(request):
         await request.post()
@@ -1501,33 +1565,30 @@ async def test_app_max_client_size_none(aiohttp_client):
     assert resp_text == 'ok'
 
 
-async def test_post_max_client_size(aiohttp_client):
+async def test_post_max_client_size(aiohttp_client) -> None:
 
     async def handler(request):
-        try:
-            await request.post()
-        except ValueError:
-            return web.Response()
-        raise web.HTTPBadRequest()
+        await request.post()
+        return web.Response()
 
     app = web.Application(client_max_size=10)
     app.router.add_post('/', handler)
     client = await aiohttp_client(app)
 
-    data = {"long_string": 1024 * 'x', 'file': io.BytesIO(b'test')}
+    data = {'long_string': 1024 * 'x', 'file': io.BytesIO(b'test')}
     resp = await client.post('/', data=data)
 
-    assert 200 == resp.status
+    assert 413 == resp.status
+    resp_text = await resp.text()
+    assert 'Maximum request body size 10 exceeded, ' \
+           'actual body size 1024' in resp_text
 
 
-async def test_post_max_client_size_for_file(aiohttp_client):
+async def test_post_max_client_size_for_file(aiohttp_client) -> None:
 
     async def handler(request):
-        try:
-            await request.post()
-        except ValueError:
-            return web.Response()
-        raise web.HTTPBadRequest()
+        await request.post()
+        return web.Response()
 
     app = web.Application(client_max_size=2)
     app.router.add_post('/', handler)
@@ -1536,10 +1597,10 @@ async def test_post_max_client_size_for_file(aiohttp_client):
     data = {'file': io.BytesIO(b'test')}
     resp = await client.post('/', data=data)
 
-    assert 200 == resp.status
+    assert 413 == resp.status
 
 
-async def test_response_with_bodypart(aiohttp_client):
+async def test_response_with_bodypart(aiohttp_client) -> None:
 
     async def handler(request):
         reader = await request.multipart()
@@ -1563,7 +1624,57 @@ async def test_response_with_bodypart(aiohttp_client):
                     {'name': 'file', 'filename': 'file', 'filename*': 'file'})
 
 
-async def test_request_clone(aiohttp_client):
+async def test_response_with_bodypart_named(aiohttp_client, tmpdir) -> None:
+
+    async def handler(request):
+        reader = await request.multipart()
+        part = await reader.next()
+        return web.Response(body=part)
+
+    app = web.Application(client_max_size=2)
+    app.router.add_post('/', handler)
+    client = await aiohttp_client(app)
+
+    f = tmpdir.join('foobar.txt')
+    f.write_text('test', encoding='utf8')
+    data = {'file': open(str(f), 'rb')}
+    resp = await client.post('/', data=data)
+
+    assert 200 == resp.status
+    body = await resp.read()
+    assert body == b'test'
+
+    disp = multipart.parse_content_disposition(
+        resp.headers['content-disposition'])
+    assert disp == (
+        'attachment',
+        {'name': 'file', 'filename': 'foobar.txt', 'filename*': 'foobar.txt'}
+    )
+
+
+async def test_response_with_bodypart_invalid_name(aiohttp_client) -> None:
+
+    async def handler(request):
+        reader = await request.multipart()
+        part = await reader.next()
+        return web.Response(body=part)
+
+    app = web.Application(client_max_size=2)
+    app.router.add_post('/', handler)
+    client = await aiohttp_client(app)
+
+    with aiohttp.MultipartWriter() as mpwriter:
+        mpwriter.append(b'test')
+        resp = await client.post('/', data=mpwriter)
+
+    assert 200 == resp.status
+    body = await resp.read()
+    assert body == b'test'
+
+    assert 'content-disposition' not in resp.headers
+
+
+async def test_request_clone(aiohttp_client) -> None:
 
     async def handler(request):
         r2 = request.clone(method='POST')
@@ -1579,7 +1690,7 @@ async def test_request_clone(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_await(aiohttp_server):
+async def test_await(aiohttp_server) -> None:
 
     async def handler(request):
         resp = web.StreamResponse(headers={'content-length': str(4)})
@@ -1605,7 +1716,7 @@ async def test_await(aiohttp_server):
         assert resp.connection is None
 
 
-async def test_response_context_manager(aiohttp_server):
+async def test_response_context_manager(aiohttp_server) -> None:
 
     async def handler(request):
         return web.Response()
@@ -1620,7 +1731,7 @@ async def test_response_context_manager(aiohttp_server):
     assert resp.connection is None
 
 
-async def test_response_context_manager_error(aiohttp_server):
+async def test_response_context_manager_error(aiohttp_server) -> None:
 
     async def handler(request):
         return web.Response(text='some text')
@@ -1657,7 +1768,8 @@ async def aiohttp_client_api_context_manager(aiohttp_server):
     assert resp.connection is None
 
 
-async def test_context_manager_close_on_release(aiohttp_server, mocker):
+async def test_context_manager_close_on_release(aiohttp_server,
+                                                mocker) -> None:
 
     async def handler(request):
         resp = web.StreamResponse()
@@ -1682,7 +1794,7 @@ async def test_context_manager_close_on_release(aiohttp_server, mocker):
         assert proto.close.called
 
 
-async def test_iter_any(aiohttp_server):
+async def test_iter_any(aiohttp_server) -> None:
 
     data = b'0123456789' * 1024
 
@@ -1702,19 +1814,19 @@ async def test_iter_any(aiohttp_server):
             assert resp.status == 200
 
 
-async def test_request_tracing(aiohttp_server):
+async def test_request_tracing(aiohttp_server) -> None:
 
-    on_request_start = mock.Mock(side_effect=asyncio.coroutine(mock.Mock()))
-    on_request_end = mock.Mock(side_effect=asyncio.coroutine(mock.Mock()))
+    on_request_start = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
+    on_request_end = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
     on_dns_resolvehost_start = mock.Mock(
-        side_effect=asyncio.coroutine(mock.Mock()))
+        side_effect=make_mocked_coro(mock.Mock()))
     on_dns_resolvehost_end = mock.Mock(
-        side_effect=asyncio.coroutine(mock.Mock()))
-    on_request_redirect = mock.Mock(side_effect=asyncio.coroutine(mock.Mock()))
+        side_effect=make_mocked_coro(mock.Mock()))
+    on_request_redirect = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
     on_connection_create_start = mock.Mock(
-        side_effect=asyncio.coroutine(mock.Mock()))
+        side_effect=make_mocked_coro(mock.Mock()))
     on_connection_create_end = mock.Mock(
-        side_effect=asyncio.coroutine(mock.Mock()))
+        side_effect=make_mocked_coro(mock.Mock()))
 
     async def redirector(request):
         raise web.HTTPFound(location=URL('/redirected'))
@@ -1777,7 +1889,7 @@ async def test_request_tracing(aiohttp_server):
     await client.close()
 
 
-async def test_return_http_exception_deprecated(aiohttp_client):
+async def test_return_http_exception_deprecated(aiohttp_client) -> None:
 
     async def handler(request):
         return web.HTTPForbidden()
@@ -1790,7 +1902,7 @@ async def test_return_http_exception_deprecated(aiohttp_client):
         await client.get('/')
 
 
-async def test_request_path(aiohttp_client):
+async def test_request_path(aiohttp_client) -> None:
 
     async def handler(request):
         assert request.path_qs == '/path%20to?a=1'
@@ -1808,7 +1920,7 @@ async def test_request_path(aiohttp_client):
     assert 'OK' == txt
 
 
-async def test_app_add_routes(aiohttp_client):
+async def test_app_add_routes(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response()
@@ -1819,3 +1931,31 @@ async def test_app_add_routes(aiohttp_client):
     client = await aiohttp_client(app)
     resp = await client.get('/get')
     assert resp.status == 200
+
+
+async def test_request_headers_type(aiohttp_client) -> None:
+
+    async def handler(request):
+        assert isinstance(request.headers, CIMultiDictProxy)
+        return web.Response()
+
+    app = web.Application()
+    app.add_routes([web.get('/get', handler)])
+
+    client = await aiohttp_client(app)
+    resp = await client.get('/get')
+    assert resp.status == 200
+
+
+async def test_signal_on_error_handler(aiohttp_client) -> None:
+
+    async def on_prepare(request, response):
+        response.headers['X-Custom'] = 'val'
+
+    app = web.Application()
+    app.on_response_prepare.append(on_prepare)
+
+    client = await aiohttp_client(app)
+    resp = await client.get('/')
+    assert resp.status == 404
+    assert resp.headers['X-Custom'] == 'val'
