@@ -1,15 +1,17 @@
 """Tests for aiohttp/server.py"""
 
 import asyncio
+import platform
 import socket
 from functools import partial
-from html import escape
 from unittest import mock
 
 import pytest
 
 from aiohttp import helpers, http, streams, web
 
+IS_MACOS = platform.system() == 'Darwin'
+
 
 @pytest.fixture
 def make_srv(loop, manager):
@@ -30,7 +32,9 @@ def make_srv(loop, manager):
 
 @pytest.fixture
 def manager(request_handler, loop):
-    return web.Server(request_handler, loop=loop)
+    async def maker():
+        return web.Server(request_handler)
+    return loop.run_until_complete(maker())
 
 
 @pytest.fixture
@@ -38,9 +42,12 @@ def srv(make_srv, transport):
     srv = make_srv()
     srv.connection_made(transport)
     transport.close.side_effect = partial(srv.connection_lost, None)
-    srv._drain_helper = mock.Mock()
-    srv._drain_helper.side_effect = helpers.noop
-    return srv
+    with mock.patch.object(
+        web.RequestHandler,
+        '_drain_helper',
+        side_effect=helpers.noop
+    ):
+        yield srv
 
 
 @pytest.fixture
@@ -98,7 +105,8 @@ def ceil(mocker):
     mocker.patch('aiohttp.helpers.ceil').side_effect = ceil
 
 
-async def test_shutdown(srv, loop, transport):
+async def test_shutdown(srv, transport) -> None:
+    loop = asyncio.get_event_loop()
     assert transport is srv.transport
 
     srv._keepalive = True
@@ -117,11 +125,11 @@ async def test_shutdown(srv, loop, transport):
     assert srv.transport is None
 
     assert not srv._task_handler
-    await asyncio.sleep(0.1, loop=loop)
+    await asyncio.sleep(0.1)
     assert task_handler.done()
 
 
-async def test_double_shutdown(srv, transport):
+async def test_double_shutdown(srv, transport) -> None:
     await srv.shutdown()
     assert transport.close.called
     assert srv.transport is None
@@ -132,7 +140,8 @@ async def test_double_shutdown(srv, transport):
     assert srv.transport is None
 
 
-async def test_shutdown_wait_error_handler(loop, srv, transport):
+async def test_shutdown_wait_error_handler(srv, transport) -> None:
+    loop = asyncio.get_event_loop()
 
     async def _error_handle():
         pass
@@ -142,14 +151,14 @@ async def test_shutdown_wait_error_handler(loop, srv, transport):
     assert srv._error_handler.done()
 
 
-async def test_close_after_response(srv, loop, transport):
+async def test_close_after_response(srv, transport) -> None:
     srv.data_received(
         b'GET / HTTP/1.0\r\n'
         b'Host: example.com\r\n'
         b'Content-Length: 0\r\n\r\n')
     h = srv._task_handler
 
-    await asyncio.sleep(0.1, loop=loop)
+    await asyncio.sleep(0.1)
     assert srv._waiter is None
     assert srv._task_handler is None
 
@@ -159,13 +168,13 @@ async def test_close_after_response(srv, loop, transport):
     assert h.done()
 
 
-def test_connection_made(make_srv):
+def test_connection_made(make_srv) -> None:
     srv = make_srv()
     srv.connection_made(mock.Mock())
     assert not srv._force_close
 
 
-def test_connection_made_with_tcp_keepaplive(make_srv, transport):
+def test_connection_made_with_tcp_keepaplive(make_srv, transport) -> None:
     srv = make_srv()
 
     sock = mock.Mock()
@@ -175,7 +184,7 @@ def test_connection_made_with_tcp_keepaplive(make_srv, transport):
                                        socket.SO_KEEPALIVE, 1)
 
 
-def test_connection_made_without_tcp_keepaplive(make_srv):
+def test_connection_made_without_tcp_keepaplive(make_srv) -> None:
     srv = make_srv(tcp_keepalive=False)
 
     sock = mock.Mock()
@@ -185,14 +194,14 @@ def test_connection_made_without_tcp_keepaplive(make_srv):
     assert not sock.setsockopt.called
 
 
-def test_eof_received(make_srv):
+def test_eof_received(make_srv) -> None:
     srv = make_srv()
     srv.connection_made(mock.Mock())
     srv.eof_received()
     # assert srv.reader._eof
 
 
-async def test_connection_lost(srv, loop):
+async def test_connection_lost(srv) -> None:
     srv.data_received(
         b'GET / HTTP/1.1\r\n'
         b'Host: example.com\r\n'
@@ -200,7 +209,7 @@ async def test_connection_lost(srv, loop):
     srv._keepalive = True
 
     handle = srv._task_handler
-    await asyncio.sleep(0, loop=loop)  # wait for .start() starting
+    await asyncio.sleep(0)  # wait for .start() starting
     srv.connection_lost(None)
 
     assert srv._force_close
@@ -210,7 +219,7 @@ async def test_connection_lost(srv, loop):
     assert not srv._task_handler
 
 
-def test_srv_keep_alive(srv):
+def test_srv_keep_alive(srv) -> None:
     assert not srv._keepalive
 
     srv.keep_alive(True)
@@ -220,7 +229,7 @@ def test_srv_keep_alive(srv):
     assert not srv._keepalive
 
 
-def test_srv_keep_alive_disable(srv):
+def test_srv_keep_alive_disable(srv) -> None:
     handle = srv._keepalive_handle = mock.Mock()
 
     srv.keep_alive(False)
@@ -229,24 +238,24 @@ def test_srv_keep_alive_disable(srv):
     handle.cancel.assert_called_with()
 
 
-async def test_simple(srv, loop, buf):
+async def test_simple(srv, buf) -> None:
     srv.data_received(
         b'GET / HTTP/1.1\r\n\r\n')
 
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0.05)
     assert buf.startswith(b'HTTP/1.1 200 OK\r\n')
 
 
-async def test_bad_method(srv, loop, buf):
+async def test_bad_method(srv, buf) -> None:
     srv.data_received(
-        b'!@#$ / HTTP/1.0\r\n'
+        b':BAD; / HTTP/1.0\r\n'
         b'Host: example.com\r\n\r\n')
 
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0)
     assert buf.startswith(b'HTTP/1.0 400 Bad Request\r\n')
 
 
-async def test_data_received_error(srv, loop, buf):
+async def test_data_received_error(srv, buf) -> None:
     transport = srv.transport
     srv._request_parser = mock.Mock()
     srv._request_parser.feed_data.side_effect = TypeError
@@ -255,57 +264,31 @@ async def test_data_received_error(srv, loop, buf):
         b'!@#$ / HTTP/1.0\r\n'
         b'Host: example.com\r\n\r\n')
 
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0)
     assert buf.startswith(b'HTTP/1.0 500 Internal Server Error\r\n')
     assert transport.close.called
     assert srv._error_handler is None
 
 
-async def test_line_too_long(srv, loop, buf):
+async def test_line_too_long(srv, buf) -> None:
     srv.data_received(b''.join([b'a' for _ in range(10000)]) + b'\r\n\r\n')
 
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0)
     assert buf.startswith(b'HTTP/1.0 400 Bad Request\r\n')
 
 
-async def test_invalid_content_length(srv, loop, buf):
+async def test_invalid_content_length(srv, buf) -> None:
     srv.data_received(
         b'GET / HTTP/1.0\r\n'
         b'Host: example.com\r\n'
         b'Content-Length: sdgg\r\n\r\n')
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0)
 
     assert buf.startswith(b'HTTP/1.0 400 Bad Request\r\n')
 
 
-async def test_handle_error__utf(
-    make_srv, buf, transport, loop, request_handler
-):
-    request_handler.side_effect = RuntimeError('что-то пошло не так')
-
-    srv = make_srv(debug=True)
-    srv.connection_made(transport)
-    srv.keep_alive(True)
-    srv.logger = mock.Mock()
-
-    srv.data_received(
-        b'GET / HTTP/1.0\r\n'
-        b'Host: example.com\r\n'
-        b'Content-Length: 0\r\n\r\n')
-    await asyncio.sleep(0, loop=loop)
-
-    assert b'HTTP/1.0 500 Internal Server Error' in buf
-    assert b'Content-Type: text/html; charset=utf-8' in buf
-    pattern = escape("RuntimeError: что-то пошло не так")
-    assert pattern.encode('utf-8') in buf
-    assert not srv._keepalive
-
-    srv.logger.exception.assert_called_with(
-        "Error handling request", exc_info=mock.ANY)
-
-
 async def test_unhandled_runtime_error(
-    make_srv, loop, transport, request_handler
+    make_srv, transport, request_handler
 ):
 
     async def handle(request):
@@ -332,7 +315,7 @@ async def test_unhandled_runtime_error(
 
 
 async def test_handle_uncompleted(
-        make_srv, loop, transport, handle_with_error, request_handler):
+        make_srv, transport, handle_with_error, request_handler):
     closed = False
 
     def close():
@@ -358,8 +341,13 @@ async def test_handle_uncompleted(
         "Error handling request", exc_info=mock.ANY)
 
 
+@pytest.mark.xfail(
+    IS_MACOS,
+    raises=TypeError,
+    reason='Intermittently fails on macOS',
+)
 async def test_handle_uncompleted_pipe(
-        make_srv, loop, transport, request_handler, handle_with_error):
+        make_srv, transport, request_handler, handle_with_error):
     closed = False
     normal_completed = False
 
@@ -376,7 +364,7 @@ async def test_handle_uncompleted_pipe(
     async def handle(request):
         nonlocal normal_completed
         normal_completed = True
-        await asyncio.sleep(0.05, loop=loop)
+        await asyncio.sleep(0.05)
         return web.Response()
 
     # normal
@@ -385,7 +373,7 @@ async def test_handle_uncompleted_pipe(
         b'GET / HTTP/1.1\r\n'
         b'Host: example.com\r\n'
         b'Content-Length: 0\r\n\r\n')
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0)
 
     # with exception
     request_handler.side_effect = handle_with_error()
@@ -396,7 +384,7 @@ async def test_handle_uncompleted_pipe(
 
     assert srv._task_handler
 
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0)
 
     await srv._task_handler
     assert normal_completed
@@ -406,161 +394,174 @@ async def test_handle_uncompleted_pipe(
         "Error handling request", exc_info=mock.ANY)
 
 
-async def test_lingering(srv, loop, transport):
+async def test_lingering(srv, transport) -> None:
     assert not transport.close.called
 
     async def handle(message, request, writer):
         pass
 
-    srv.handle_request = handle
-    srv.data_received(
-        b'GET / HTTP/1.0\r\n'
-        b'Host: example.com\r\n'
-        b'Content-Length: 3\r\n\r\n')
+    with mock.patch.object(
+        web.RequestHandler, 'handle_request', create=True, new=handle
+    ):
+        srv.data_received(
+            b'GET / HTTP/1.0\r\n'
+            b'Host: example.com\r\n'
+            b'Content-Length: 3\r\n\r\n')
 
-    await asyncio.sleep(0.05, loop=loop)
-    assert not transport.close.called
+        await asyncio.sleep(0.05)
+        assert not transport.close.called
 
-    srv.data_received(b'123')
+        srv.data_received(b'123')
 
-    await asyncio.sleep(0, loop=loop)
-    transport.close.assert_called_with()
+        await asyncio.sleep(0)
+        transport.close.assert_called_with()
 
 
-async def test_lingering_disabled(make_srv, loop, transport, request_handler):
+async def test_lingering_disabled(make_srv,
+                                  transport, request_handler) -> None:
 
     async def handle_request(request):
-        await asyncio.sleep(0, loop=loop)
+        await asyncio.sleep(0)
 
     srv = make_srv(lingering_time=0)
     srv.connection_made(transport)
     request_handler.side_effect = handle_request
 
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0)
     assert not transport.close.called
 
     srv.data_received(
         b'GET / HTTP/1.0\r\n'
         b'Host: example.com\r\n'
         b'Content-Length: 50\r\n\r\n')
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0)
     assert not transport.close.called
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0.05)
     transport.close.assert_called_with()
 
 
 async def test_lingering_timeout(
-    make_srv, loop, transport, ceil, request_handler
+    make_srv, transport, ceil, request_handler
 ):
 
     async def handle_request(request):
-        await asyncio.sleep(0, loop=loop)
+        await asyncio.sleep(0)
 
     srv = make_srv(lingering_time=1e-30)
     srv.connection_made(transport)
     request_handler.side_effect = handle_request
 
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0.05)
     assert not transport.close.called
 
     srv.data_received(
         b'GET / HTTP/1.0\r\n'
         b'Host: example.com\r\n'
         b'Content-Length: 50\r\n\r\n')
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0)
     assert not transport.close.called
 
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0.05)
     transport.close.assert_called_with()
 
 
-def test_handle_cancel(make_srv, loop, transport):
+async def test_handle_payload_access_error(
+    make_srv, transport, request_handler
+):
+    srv = make_srv(lingering_time=0)
+    srv.connection_made(transport)
+    srv.data_received(
+        b'POST /test HTTP/1.1\r\n'
+        b'Content-Length: 9\r\n\r\n'
+        b'some data'
+    )
+    # start request_handler task
+    await asyncio.sleep(0.05)
+
+    with pytest.raises(web.PayloadAccessError):
+        await request_handler.call_args[0][0].content.read()
+
+
+async def test_handle_cancel(make_srv, transport) -> None:
     log = mock.Mock()
 
     srv = make_srv(logger=log, debug=True)
     srv.connection_made(transport)
 
     async def handle_request(message, payload, writer):
-        await asyncio.sleep(10, loop=loop)
-
-    srv.handle_request = handle_request
+        await asyncio.sleep(10)
 
     async def cancel():
         srv._task_handler.cancel()
 
-    srv.data_received(
-        b'GET / HTTP/1.0\r\n'
-        b'Content-Length: 10\r\n'
-        b'Host: example.com\r\n\r\n')
+    with mock.patch.object(
+        web.RequestHandler, 'handle_request', create=True, new=handle_request
+    ):
+        srv.data_received(
+            b'GET / HTTP/1.0\r\n'
+            b'Content-Length: 10\r\n'
+            b'Host: example.com\r\n\r\n')
 
-    loop.run_until_complete(
-        asyncio.gather(srv._task_handler, cancel(), loop=loop))
-    assert log.debug.called
+        await asyncio.gather(srv._task_handler, cancel())
+        assert log.debug.called
 
 
-def test_handle_cancelled(make_srv, loop, transport):
+async def test_handle_cancelled(make_srv, transport) -> None:
     log = mock.Mock()
 
     srv = make_srv(logger=log, debug=True)
     srv.connection_made(transport)
 
-    srv.handle_request = mock.Mock()
     # start request_handler task
-    loop.run_until_complete(asyncio.sleep(0, loop=loop))
+    await asyncio.sleep(0)
 
     srv.data_received(
         b'GET / HTTP/1.0\r\n'
         b'Host: example.com\r\n\r\n')
 
     r_handler = srv._task_handler
-    assert loop.run_until_complete(r_handler) is None
+    assert (await r_handler) is None
 
 
-async def test_handle_400(srv, loop, buf, transport):
+async def test_handle_400(srv, buf, transport) -> None:
     srv.data_received(b'GET / HT/asd\r\n\r\n')
 
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0)
     assert b'400 Bad Request' in buf
 
 
-def test_handle_500(srv, loop, buf, transport, request_handler):
-    request_handler.side_effect = ValueError
-
-    srv.data_received(
-        b'GET / HTTP/1.0\r\n'
-        b'Host: example.com\r\n\r\n')
-    loop.run_until_complete(srv._task_handler)
-
-    assert b'500 Internal Server Error' in buf
-
-
-async def test_keep_alive(make_srv, loop, transport, ceil):
+async def test_keep_alive(make_srv, transport, ceil) -> None:
+    loop = asyncio.get_event_loop()
     srv = make_srv(keepalive_timeout=0.05)
-    srv.KEEPALIVE_RESCHEDULE_DELAY = 0.1
-    srv.connection_made(transport)
+    future = loop.create_future()
+    future.set_result(1)
 
-    srv.keep_alive(True)
-    srv.handle_request = mock.Mock()
-    srv.handle_request.return_value = loop.create_future()
-    srv.handle_request.return_value.set_result(1)
+    with mock.patch.object(
+        web.RequestHandler, 'KEEPALIVE_RESCHEDULE_DELAY', new=0.1
+    ), mock.patch.object(
+        web.RequestHandler, 'handle_request', create=True, return_value=future
+    ):
+        srv.connection_made(transport)
+        srv.keep_alive(True)
+        srv.data_received(
+            b'GET / HTTP/1.1\r\n'
+            b'Host: example.com\r\n'
+            b'Content-Length: 0\r\n\r\n')
 
-    srv.data_received(
-        b'GET / HTTP/1.1\r\n'
-        b'Host: example.com\r\n'
-        b'Content-Length: 0\r\n\r\n')
+        waiter = None
+        while waiter is None:
+            await asyncio.sleep(0)
+            waiter = srv._waiter
+        assert srv._keepalive_handle is not None
+        assert not transport.close.called
 
-    await asyncio.sleep(0, loop=loop)
-    waiter = srv._waiter
-    assert waiter
-    assert srv._keepalive_handle is not None
-    assert not transport.close.called
-
-    await asyncio.sleep(0.2, loop=loop)
-    assert transport.close.called
-    assert waiter.cancelled
+        await asyncio.sleep(0.2)
+        assert transport.close.called
+        assert waiter.cancelled
 
 
-def test_srv_process_request_without_timeout(make_srv, loop, transport):
+async def test_srv_process_request_without_timeout(make_srv,
+                                                   transport) -> None:
     srv = make_srv()
     srv.connection_made(transport)
 
@@ -568,24 +569,25 @@ def test_srv_process_request_without_timeout(make_srv, loop, transport):
         b'GET / HTTP/1.0\r\n'
         b'Host: example.com\r\n\r\n')
 
-    loop.run_until_complete(srv._task_handler)
+    await srv._task_handler
     assert transport.close.called
 
 
-def test_keep_alive_timeout_default(srv):
+def test_keep_alive_timeout_default(srv) -> None:
     assert 75 == srv.keepalive_timeout
 
 
-def test_keep_alive_timeout_nondefault(make_srv):
+def test_keep_alive_timeout_nondefault(make_srv) -> None:
     srv = make_srv(keepalive_timeout=10)
     assert 10 == srv.keepalive_timeout
 
 
-async def test_supports_connect_method(srv, loop, transport, request_handler):
+async def test_supports_connect_method(srv,
+                                       transport, request_handler) -> None:
     srv.data_received(
         b'CONNECT aiohttp.readthedocs.org:80 HTTP/1.0\r\n'
         b'Content-Length: 0\r\n\r\n')
-    await asyncio.sleep(0.1, loop=loop)
+    await asyncio.sleep(0.1)
 
     assert request_handler.called
     assert isinstance(
@@ -593,18 +595,18 @@ async def test_supports_connect_method(srv, loop, transport, request_handler):
         streams.StreamReader)
 
 
-async def test_content_length_0(srv, loop, request_handler):
+async def test_content_length_0(srv, request_handler) -> None:
     srv.data_received(
         b'GET / HTTP/1.1\r\n'
         b'Host: example.org\r\n'
         b'Content-Length: 0\r\n\r\n')
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0)
 
     assert request_handler.called
     assert request_handler.call_args[0][0].content == streams.EMPTY_PAYLOAD
 
 
-def test_rudimentary_transport(srv, loop):
+def test_rudimentary_transport(srv) -> None:
     transport = mock.Mock()
     srv.connection_made(transport)
 
@@ -627,37 +629,43 @@ def test_rudimentary_transport(srv, loop):
     assert not srv._reading_paused
 
 
-async def test_close(srv, loop, transport):
+async def test_close(srv, transport) -> None:
     transport.close.side_effect = partial(srv.connection_lost, None)
     srv.connection_made(transport)
+    await asyncio.sleep(0)
 
-    srv.handle_request = mock.Mock()
-    srv.handle_request.side_effect = helpers.noop
+    handle_request = mock.Mock()
+    handle_request.side_effect = helpers.noop
+    with mock.patch.object(
+        web.RequestHandler,
+        'handle_request',
+        create=True,
+        new=handle_request
+    ):
+        assert transport is srv.transport
 
-    assert transport is srv.transport
+        srv._keepalive = True
+        srv.data_received(
+            b'GET / HTTP/1.1\r\n'
+            b'Host: example.com\r\n'
+            b'Content-Length: 0\r\n\r\n'
+            b'GET / HTTP/1.1\r\n'
+            b'Host: example.com\r\n'
+            b'Content-Length: 0\r\n\r\n')
 
-    srv._keepalive = True
-    srv.data_received(
-        b'GET / HTTP/1.1\r\n'
-        b'Host: example.com\r\n'
-        b'Content-Length: 0\r\n\r\n'
-        b'GET / HTTP/1.1\r\n'
-        b'Host: example.com\r\n'
-        b'Content-Length: 0\r\n\r\n')
+        await asyncio.sleep(0.05)
+        assert srv._task_handler
+        assert srv._waiter
 
-    await asyncio.sleep(0, loop=loop)
-    assert srv._task_handler
-    assert srv._waiter
-
-    srv.close()
-    await asyncio.sleep(0, loop=loop)
-    assert srv._task_handler is None
-    assert srv.transport is None
-    assert transport.close.called
+        srv.close()
+        await asyncio.sleep(0)
+        assert srv._task_handler is None
+        assert srv.transport is None
+        assert transport.close.called
 
 
 async def test_pipeline_multiple_messages(
-    srv, loop, transport, request_handler
+    srv, transport, request_handler
 ):
     transport.close.side_effect = partial(srv.connection_lost, None)
 
@@ -685,14 +693,14 @@ async def test_pipeline_multiple_messages(
     assert len(srv._messages) == 2
     assert srv._waiter is not None
 
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0.05)
     assert srv._task_handler is not None
     assert srv._waiter is not None
     assert processed == 2
 
 
 async def test_pipeline_response_order(
-    srv, loop, buf, transport, request_handler
+    srv, buf, transport, request_handler
 ):
     transport.close.side_effect = partial(srv.connection_lost, None)
     srv._keepalive = True
@@ -701,7 +709,7 @@ async def test_pipeline_response_order(
 
     async def handle1(request):
         nonlocal processed
-        await asyncio.sleep(0.01, loop=loop)
+        await asyncio.sleep(0.01)
         resp = web.StreamResponse()
         await resp.prepare(request)
         await resp.write(b'test1')
@@ -714,7 +722,7 @@ async def test_pipeline_response_order(
         b'GET / HTTP/1.1\r\n'
         b'Host: example.com\r\n'
         b'Content-Length: 0\r\n\r\n')
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0)
 
     # second
 
@@ -732,15 +740,15 @@ async def test_pipeline_response_order(
         b'GET / HTTP/1.1\r\n'
         b'Host: example.com\r\n'
         b'Content-Length: 0\r\n\r\n')
-    await asyncio.sleep(0, loop=loop)
+    await asyncio.sleep(0)
 
     assert srv._task_handler is not None
 
-    await asyncio.sleep(0.1, loop=loop)
+    await asyncio.sleep(0.1)
     assert processed == [1, 2]
 
 
-def test_data_received_close(srv):
+def test_data_received_close(srv) -> None:
     srv.close()
     srv.data_received(
         b'GET / HTTP/1.1\r\n'
@@ -750,7 +758,7 @@ def test_data_received_close(srv):
     assert not srv._messages
 
 
-def test_data_received_force_close(srv):
+def test_data_received_force_close(srv) -> None:
     srv.force_close()
     srv.data_received(
         b'GET / HTTP/1.1\r\n'
@@ -760,7 +768,8 @@ def test_data_received_force_close(srv):
     assert not srv._messages
 
 
-async def test__process_keepalive(loop, srv):
+async def test__process_keepalive(srv) -> None:
+    loop = asyncio.get_event_loop()
     # wait till the waiter is waiting
     await asyncio.sleep(0)
 
@@ -775,7 +784,8 @@ async def test__process_keepalive(loop, srv):
         assert srv._force_close
 
 
-async def test__process_keepalive_schedule_next(loop, srv):
+async def test__process_keepalive_schedule_next(srv) -> None:
+    loop = asyncio.get_event_loop()
     # wait till the waiter is waiting
     await asyncio.sleep(0)
 
@@ -792,8 +802,60 @@ async def test__process_keepalive_schedule_next(loop, srv):
             )
 
 
-def test__process_keepalive_force_close(loop, srv):
+async def test__process_keepalive_force_close(srv) -> None:
+    loop = asyncio.get_event_loop()
     srv._force_close = True
     with mock.patch.object(loop, "call_at") as call_at_patched:
         srv._process_keepalive()
         assert not call_at_patched.called
+
+
+async def test_two_data_received_without_waking_up_start_task(srv) -> None:
+    # make a chance to srv.start() method start waiting for srv._waiter
+    await asyncio.sleep(0.01)
+    assert srv._waiter is not None
+
+    srv.data_received(
+        b'GET / HTTP/1.1\r\n'
+        b'Host: ex.com\r\n'
+        b'Content-Length: 1\r\n\r\n'
+        b'a')
+    srv.data_received(
+        b'GET / HTTP/1.1\r\n'
+        b'Host: ex.com\r\n'
+        b'Content-Length: 1\r\n\r\n'
+        b'b')
+
+    assert len(srv._messages) == 2
+    assert srv._waiter.done()
+    await asyncio.sleep(0.01)
+
+
+async def test_client_disconnect(aiohttp_server) -> None:
+
+    async def handler(request):
+        await request.content.read(10)
+        return web.Response()
+
+    logger = mock.Mock()
+    app = web.Application()
+    app._debug = True
+    app.router.add_route('POST', '/', handler)
+    server = await aiohttp_server(app, logger=logger)
+
+    if helpers.PY_38:
+        writer = await asyncio.connect('127.0.0.1', server.port)
+    else:
+        _, writer = await asyncio.open_connection('127.0.0.1', server.port)
+    writer.write("""POST / HTTP/1.1\r
+Connection: keep-alive\r
+Content-Length: 10\r
+Host: localhost:{port}\r
+\r
+""".format(port=server.port).encode("ascii"))
+    await writer.drain()
+    await asyncio.sleep(0.1)
+    writer.write(b"x")
+    writer.close()
+    await asyncio.sleep(0.1)
+    logger.debug.assert_called_with('Ignored premature client disconnection 2')
