@@ -109,3 +109,23 @@ def test_gzip_if_header_present_and_file_not_available(loop) -> None:
 
     assert filepath.open.called
     assert not gz_filepath.open.called
+
+
+def test_status_controlled_by_user(loop) -> None:
+    request = make_mocked_request(
+        'GET', 'http://python.org/logo.png', headers={
+        }
+    )
+
+    filepath = mock.Mock()
+    filepath.name = 'logo.png'
+    filepath.open = mock.mock_open()
+    filepath.stat.return_value = mock.MagicMock()
+    filepath.stat.st_size = 1024
+
+    file_sender = FileResponse(filepath, status=203)
+    file_sender._sendfile = make_mocked_coro(None)
+
+    loop.run_until_complete(file_sender.prepare(request))
+
+    assert file_sender._status == 203
